@@ -29,13 +29,13 @@
  *          
  * \since   2005/07/02
  *
- * \date    \$Date: 2007/05/10 09:22:14 $
+ * \date    \$Date: 2007/06/06 13:35:17 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.81 $
+ * \version \$Revision: 1.86 $
  */
 
 
@@ -47,7 +47,7 @@
 
 
 
-
+ 
 
 /******************************************************************************
  * Headers
@@ -68,6 +68,7 @@ using std::vector;
 class ASTE;
 class pPartnerLink;
 class WSDL_PartnerLinkType;
+class BPEL4Chor_messageLink;
 
 
 
@@ -86,7 +87,7 @@ typedef enum
   FHCF = 1,		///< control flow in fault handler
   CHCF = 2,		///< control flow in compensation handler
   THCF = 3		///< control flow in termination handler
-} controlFlowType;
+} ENUM_control_flow_type;
 
 
 /*!
@@ -100,7 +101,7 @@ typedef enum
   T_PATTERN,		///< for correlation: "request", "response" or "request-response"
   T_BPELVARIABLENAME,	///< a variable name (NCName without ".")
   T_UINT		///< unsigned integers
-} attributeType;
+} ENUM_attribute_type;
 
 
 
@@ -118,41 +119,71 @@ typedef enum
 class ASTE
 {
   private:
-    unsigned int id;			///< the id of the AST phylum
-    unsigned int type;			///< the type of the node, identified by the keywords as parsed by Flex
+    /// the identifier of the AST phylum
+    unsigned int id;
+
+    ///< the type of the node, identified by the keywords as parsed by Flex
+    unsigned int type;
+
 
     void checkRequiredAttributes(string required[], unsigned int length);
     void setStandardAttributes(string names[], string values[], unsigned int length);
-    void checkAttributeType(string attribute, attributeType type);
+    void checkAttributeType(string attribute, ENUM_attribute_type type);
+
 
   public:
-    kc::impl_abstract_phylum* phylum;	///< the AST phylum
+    /// a pointer to the phylum of the AST this object is assigned to; this
+    /// pointer is set by the constructor macro CTOR.
+    kc::impl_abstract_phylum* phylum;
 
-    map<string, string> attributes;	///< the parsed attributes
 
-    unsigned int parentScopeId;		///< the identifier of the parent scope
-    unsigned int parentActivityId;	///< the identifier of the parent activity
+    /**************************************************************************
+     * Attributes
+     *************************************************************************/
+
+    /// a name/value-mapping of the attributes;
+    /// note: the attribute "referenceLine" is added by the parser and filled
+    /// with the line number of the parsed element
+    map<string, string> attributes;
+
+    /// the name of a channel associated to a communication activity
+    string channelName;
+
+    /// the name of a second channel associated to a communication activity
+    /// (for synchronous invoke)
+    string channelName2;
+
+    /// the name of a variable associated to a communication activity
+    string variableName;
+
+    /// the name of an input variable associated to a communication activity
+    /// (for invoke)
+    string inputVariableName;
+
+    /// the name of an output variable associated to a communication activity
+    /// (for invoke)
+    string outputVariableName;
+   
+
+    /**************************************************************************
+     * Hierarchy
+     *************************************************************************/
+
+    /// the identifier of the parent scope
+    unsigned int parentScopeId;
+
+    /// the identifier of the parent activity
+    unsigned int parentActivityId;
+
+    set<unsigned int> peerScopes;	///< as defined on page 132 of the spec
+    set<unsigned int> enclosedScopes;	///< as defined on page 122 of the spec
+
+    set<unsigned int> enclosedActivities;       ///< set of all the enclosed activities
     
-    controlFlowType controlFlow;	///< the kind of control flow the activity is embedded in
 
-    string channelName;			///< name of a channel
-    string channelName2;		///< name of a channel (for synchronous invoke)
-    string variableName;		///< (unique) name of a variable
-    string inputVariableName;		///< (unique) name of an input variable
-    string outputVariableName;		///< (unique) name of an output variable
-    string linkName;			///< (unique) name of a link
-    string visConnection;		///< (unique) name of a connectionnode for the activity diagram
-    string secVisConnection;		///< (unique) name of a connectionnode for the activity diagram
-
-    pPartnerLink * plRoleDetails;        ///< additional information (myRole, partnerRole) for role-based matching of PartnerLinks
-
-    bool hasEH;				///< true if <process> or <scope> have <eventHandlers>
-    bool hasCatchAll;			///< true if <faultHandlers> have a <catchAll> branch
-    bool isStartActivity;		///< true if "createInstance" attribute is set to "yes"
-    bool cyclic;			///< true if OnAlarm has RepeatEvery
-    bool highlighted;			///< true if the activtity is to be highlighted in visualization
-    bool isUserDefined;			///< true if construct is user-defined
-    bool callable;                      ///< true if a handler actually can be called
+    /**************************************************************************
+     * Links
+     *************************************************************************/
 
     set<unsigned int> sourceLinks;	///< outgoing links
     set<unsigned int> targetLinks;	///< ingoing links
@@ -160,32 +191,63 @@ class ASTE
     unsigned int sourceActivity;	///< id of the source of a <link>
     unsigned int targetActivity;	///< id of the target of a <link>
 
-
-    unsigned int max_occurrences;	///< number of maximum occurrences of the activity
-    unsigned int max_loops;		///< number of maximum loops of the repeatable construct
-
-    set<unsigned int> peerScopes;	///< as defined on page 132 of the spec
-    set<unsigned int> enclosedScopes;	///< as defined on page 122 of the spec
+    string linkName;			///< (unique) name of a link
 
     set<unsigned int> enclosedSourceLinks;	///< the identifiers of all (recursively) enclosed source links
     set<unsigned int> enclosedTargetLinks;	///< the identifiers of all (recursively) enclosed target links
 
-    set<string> possiblyTriggeredFaults;        ///< for a scope we remember the faults that can be triggered inside
 
+    /**************************************************************************
+     * Visualization
+     *************************************************************************/
+
+    string visConnection;		///< (unique) name of a connectionnode for the activity diagram
+    string secVisConnection;		///< (unique) name of a connectionnode for the activity diagram
+    bool drawn;
+    bool highlighted;			///< true if the activtity is to be highlighted in visualization
+
+
+    /**************************************************************************
+     * Handlers
+     *************************************************************************/
+
+    set<string> possiblyTriggeredFaults;        ///< for a scope we remember the faults that can be triggered inside
     unsigned int enclosedFH;		///< the id of the enclosed fault handlers
     unsigned int enclosedCH;		///< the id of the enclosed compensation handler
-   
-    set<string> sharedCorrelationSets;	///< the identifiers of the joined correlation sets in case activity is a start activity
+    bool callable;                      ///< true if a handler actually can be called
+    bool hasCompensate;                 ///< true if a scope has an compensate
+    bool hasEH;				///< true if <process> or <scope> have <eventHandlers>
+    bool hasCatchAll;			///< true if <faultHandlers> have a <catchAll> branch
+    bool isUserDefined;			///< true if construct is user-defined
     set<string> catches;		///< the <catch> branches of <faultHandlers> to detect duplicates
+    ENUM_control_flow_type controlFlow;	///< the kind of control flow the activity is embedded in
 
-    set<unsigned int> enclosedActivities;       ///< set of all the enclosed activities
 
-    unsigned int channel_instances;
-    unsigned int channel_unique_instances;
+
+    pPartnerLink * plRoleDetails;        ///< additional information (myRole, partnerRole) for role-based matching of PartnerLinks
+
+    bool isStartActivity;		///< true if "createInstance" attribute is set to "yes"
+    bool cyclic;			///< true if OnAlarm has RepeatEvery
+
+    unsigned int max_occurrences;	///< number of maximum occurrences of the activity
+    unsigned int max_loops;		///< number of maximum loops of the repeatable construct
+  
+    set<string> sharedCorrelationSets;	///< the identifiers of the joined correlation sets in case activity is a start activity
+
+    unsigned int channel_instances; ///< an iterator for BPEL4Chor
+
+    WSDL_PartnerLinkType* partnerLinkType; ///< a link to a partnerLinkType object holding more information; used for partnerLinks
+    BPEL4Chor_messageLink* messageLink; ///< a BPEL4Chor message link (if present)
+    
+
+    /**************************************************************************
+     * Class methods
+     *************************************************************************/
 
     string createChannel(bool synchronousCommunication = false);
 
     string checkVariable(string attributename);
+    string checkMessageExchange();
     string checkLink();
     string checkPartnerLink();
     string checkCorrelationSet();
@@ -201,6 +263,7 @@ class ASTE
 
     string defineCorrelationSet();
     string defineVariable();
+    string defineMessageExchange();
     string defineLink();
     string definePartnerLink();
 
@@ -213,11 +276,16 @@ class ASTE
     bool structured();
 
     void output();
-    bool drawn;
 
-    WSDL_PartnerLinkType* partnerLinkType; ///< a link to a partnerLinkType object holding more information; used for partnerLinks
 
+    /**************************************************************************
+     * Class organization
+     *************************************************************************/
+
+    /// constructor
     ASTE(unsigned int myid, unsigned int mytype);
+
+    /// destructor
     virtual ~ASTE();
 };
 
