@@ -25,17 +25,17 @@
  *
  * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
  *          Christian Gierds <gierds@informatik.hu-berlin.de>,
- *          last changes of: \$Author: gierds $
+ *          last changes of: \$Author: nielslohmann $
  *
  * \since   2006/02/08
  *
- * \date    \$Date: 2007/04/17 11:54:08 $
+ * \date    \$Date: 2007/06/05 15:04:00 $
  *
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.69 $
+ * \version \$Revision: 1.71 $
  *
  * \ingroup debug
  * \ingroup creation
@@ -44,6 +44,11 @@
 
 
 
+
+/******************************************************************************
+ * Headers
+ *****************************************************************************/
+
 #include <cmath>
 #include <cassert>
 #include <utility>
@@ -54,10 +59,9 @@
 #include "ast-tools.h"
 #include "globals.h"
 #include "debug.h"
+#include "petrinet.h"
 
 using std::pair;
-using std::cerr;
-using std::endl;
 
 
 
@@ -397,15 +401,15 @@ void enterConsecutiveActivities( unsigned int a, unsigned int b )
 void check_SA00070( unsigned int id )
 {
   ENTER("check_SA00070");
-
+  
   // go through all enclosed activities and look if there have sources or targets
   for ( set< unsigned int >::iterator activities = globals::ASTEmap[ id ]->enclosedActivities.begin();
-          activities != globals::ASTEmap[ id ]->enclosedActivities.end();
-          activities++ )
+        activities != globals::ASTEmap[ id ]->enclosedActivities.end();
+        activities++ )
   {
     // whether it's source or target doesn't matter - the link must be defined inside the element that is checked
     set< unsigned int > links = setUnion( globals::ASTEmap[ *activities ]->sourceLinks, globals::ASTEmap[ *activities ]->targetLinks );
-
+    
     // so check every source and target
     for ( set< unsigned int>::iterator link = links.begin(); link != links.end(); link++ )
     {
@@ -562,7 +566,7 @@ void listAttributes ( unsigned int id )
              attribute->first == "surpressJoinFailure" ||
              attribute->first == "validate")
             && attribute->second == "no" )
-      )
+         )
     {
       result += " " + attribute->first + "=\""+ attribute->second +"\"";
     }
@@ -606,3 +610,15 @@ void next_index(vector<unsigned int> &current_index, const vector<unsigned int>
       current_index[i] = 1;
   }
 }
+
+
+
+
+/*!
+ * \todo Move me to the right place.
+ */
+void listAttributes ( kc::integer id )
+{
+  assert(id!=NULL);
+  listAttributes ( id->value );
+}
