@@ -18,7 +18,7 @@
  * with GNU BPEL2oWFN; see file COPYING. if not, write to the Free Software  *
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA. *
 \*****************************************************************************/
-
+ 
 /*!
  * \file    debug.cc
  *
@@ -30,13 +30,13 @@
  *
  * \since   2005/11/09
  *          
- * \date    \$Date: 2007/05/11 14:59:47 $
+ * \date    \$Date: 2007/06/04 08:02:02 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.99 $
+ * \version \$Revision: 1.105 $
  *
  * \ingroup debug
  */
@@ -147,6 +147,8 @@ void show_process_information_header()
 
 
 
+
+
 /*!
  * \brief print information about the proces
  */
@@ -225,7 +227,7 @@ void show_process_information()
     cerr << "Found " << globals::other_errors << " further errors." << endl;
   }
 
-  if (modus == M_PETRINET || modus == M_CONSISTENCY)
+  if (modus == M_PETRINET || modus == M_CHOREOGRAPHY)
     cerr << "------------------------------------------------------------------------------" << endl;
 }
 
@@ -298,6 +300,7 @@ int frontend_error(const char *msg)
 
 
 
+
 /*!
  * \brief prints a generic error message
  *
@@ -513,6 +516,10 @@ void genericError(unsigned int code, string information, string line, error_leve
     case(137): // BPEL4Chor XMLNS not found (either in BPEL4Chor file or in process)
 	{ cerr << information << endl;
 	  break; }
+
+    case(138): // no "partnerLink" or "operation" attribute found -- using BPEL4Chor attributes instead
+        { cerr << "using identifier `" << information << "' as a channel name; use BPEL4Chor topology for exact channel names" << endl;
+	  break; }
   }
 
   cerr << endl;
@@ -645,6 +652,10 @@ void SAerror(unsigned int code, string information, int lineNumber)
       { cerr << "<correlationSet> `" << information << "' already defined in this scope/process" << endl;
 	break; }
 
+    case(48):
+      { cerr << "<" << information << "> is using a variable of a wrong messageType" << endl;
+	break; }
+
     case(51):
       { cerr << "<invoke> must not be used with an inputVariable AND <toPart> elements" << endl;
 	break; }
@@ -665,10 +676,22 @@ void SAerror(unsigned int code, string information, int lineNumber)
       { cerr << "all start activities must share a correlation set with `initiate' set to \"join\"" << endl;
 	break; }
 
+    case(58):
+      { cerr << "<" << information << "> is using a variable of a wrong messageType" << endl;
+	break; }
+
     case(59):
       { cerr << "<reply> must not be used with a variable AND <toPart> elements" << endl;
 	break; }
 
+    case(60):
+      { cerr << "ould not find an explicit corresponding incoming message activity for the <reply>" << endl;
+	break; }
+
+    case(61):
+      { cerr << "messageExchange \"" << information << "\" is not defined in any enclosing scope" << endl;
+	break; }
+
     case(62):
       { cerr << "<pick> start activity must only contain <onMessage> branches" << endl;
 	break; }
@@ -761,6 +784,10 @@ void SAerror(unsigned int code, string information, int lineNumber)
        { cerr << "partnerLink reference `" << information << "' of <onEvent> cannot be resolved" << endl;
 	 break; }
 
+    case(87):
+      { cerr << "<" << information << "> is using a variable of a wrong messageType" << endl;
+	break; }
+
     case(88):
        { cerr << "<correlation> reference `" << information << "' of <onEvent> cannot be resolved" << endl;
 	 break; }
@@ -804,6 +831,7 @@ void SAerror(unsigned int code, string information, string lineNumber)
 
 
 
+
 /******************************************************************************
  * Error handling functions
  *****************************************************************************/
