@@ -349,11 +349,11 @@
  *          Partner Link Type" (http://docs.oasis-open.org/wsbpel/2.0/plnktype)
  *
  * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
- *          last changes of: \$Author: nielslohmann $
+ *          last changes of: \$Author: znamirow $
  *
  * \since   2007/04/29
  *
- * \date    \$Date: 2007/05/06 15:48:28 $
+ * \date    \$Date: 2007/05/16 11:52:39 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
@@ -363,7 +363,7 @@
  *          frontend-parser-chor.yy.
  *          See http://www.gnu.org/software/bison/bison.html for details
  *
- * \version \$Revision: 1.8 $
+ * \version \$Revision: 1.9 $
  *
  * \ingroup frontend
  */
@@ -760,11 +760,11 @@ static const yytype_int16 yyrhs[] =
 static const yytype_uint16 yyrline[] =
 {
        0,   161,   161,   173,   175,   179,   188,   190,   195,   197,
-     205,   207,   212,   211,   218,   219,   223,   232,   234,   239,
-     238,   245,   246,   251,   250,   256,   257,   261,   262,   263,
-     267,   272,   277,   286,   288,   292,   301,   303,   307,   311,
-     313,   317,   324,   326,   331,   332,   333,   342,   344,   349,
-     348,   355,   356,   360,   363,   362,   373,   375
+     205,   207,   212,   211,   218,   223,   227,   236,   238,   243,
+     242,   249,   250,   255,   254,   260,   261,   265,   266,   267,
+     271,   276,   281,   290,   292,   296,   305,   307,   311,   315,
+     317,   321,   328,   330,   335,   336,   337,   346,   348,   353,
+     352,   359,   360,   364,   367,   366,   377,   379
 };
 #endif
 
@@ -1825,94 +1825,102 @@ yyreduce:
     { globals::WSDLInfo.messages[temp_message->name] = temp_message; }
     break;
 
+  case 14:
+#line 219 "frontend-parser-wsdl.yy"
+    { 
+    temp_message->element.first = globals::tempAttributes["name"];
+    temp_message->element.second = globals::tempAttributes["element"];
+  }
+    break;
+
   case 16:
-#line 224 "frontend-parser-wsdl.yy"
-    { temp_message->parts[globals::tempAttributes["name"]] = globals::tempAttributes["type"]; }
+#line 228 "frontend-parser-wsdl.yy"
+    { temp_message->parts[globals::tempAttributes["name"]] = globals::tempAttributes["type"];}
     break;
 
   case 19:
-#line 239 "frontend-parser-wsdl.yy"
+#line 243 "frontend-parser-wsdl.yy"
     { temp_portType = new WSDL_PortType(globals::tempAttributes["name"]); }
     break;
 
   case 20:
-#line 241 "frontend-parser-wsdl.yy"
+#line 245 "frontend-parser-wsdl.yy"
     { globals::WSDLInfo.portTypes[temp_portType->name] = temp_portType; }
     break;
 
   case 23:
-#line 251 "frontend-parser-wsdl.yy"
+#line 255 "frontend-parser-wsdl.yy"
     { temp_portType->addOperation(globals::tempAttributes["name"]); }
     break;
 
   case 30:
-#line 268 "frontend-parser-wsdl.yy"
+#line 272 "frontend-parser-wsdl.yy"
     { temp_portType->addOperationDetails("input", globals::tempAttributes["message"]); }
     break;
 
   case 31:
-#line 273 "frontend-parser-wsdl.yy"
+#line 277 "frontend-parser-wsdl.yy"
     { temp_portType->addOperationDetails("output", globals::tempAttributes["message"]); }
     break;
 
   case 32:
-#line 278 "frontend-parser-wsdl.yy"
+#line 282 "frontend-parser-wsdl.yy"
     { temp_portType->addOperationDetails("fault", globals::tempAttributes["message"], globals::tempAttributes["name"]); }
     break;
 
   case 35:
-#line 293 "frontend-parser-wsdl.yy"
+#line 297 "frontend-parser-wsdl.yy"
     { globals::wsdl_information.bindings++; }
     break;
 
   case 38:
-#line 308 "frontend-parser-wsdl.yy"
+#line 312 "frontend-parser-wsdl.yy"
     { globals::wsdl_information.services++; }
     break;
 
   case 43:
-#line 327 "frontend-parser-wsdl.yy"
+#line 331 "frontend-parser-wsdl.yy"
     { globals::wsdl_information.properties++; }
     break;
 
   case 46:
-#line 334 "frontend-parser-wsdl.yy"
+#line 338 "frontend-parser-wsdl.yy"
     { genericError(125, globals::last_error_token, globals::last_error_line, ERRORLEVEL_NOTICE); }
     break;
 
   case 49:
-#line 349 "frontend-parser-wsdl.yy"
+#line 353 "frontend-parser-wsdl.yy"
     { temp_partnerLinkType = new WSDL_PartnerLinkType(globals::tempAttributes["name"]); }
     break;
 
   case 50:
-#line 351 "frontend-parser-wsdl.yy"
+#line 355 "frontend-parser-wsdl.yy"
     { globals::WSDLInfo.partnerLinkTypes[temp_partnerLinkType->name] = temp_partnerLinkType; }
     break;
 
   case 53:
-#line 361 "frontend-parser-wsdl.yy"
+#line 365 "frontend-parser-wsdl.yy"
     { temp_partnerLinkType->addRole(globals::tempAttributes["name"], globals::tempAttributes["portType"]); }
     break;
 
   case 54:
-#line 363 "frontend-parser-wsdl.yy"
+#line 367 "frontend-parser-wsdl.yy"
     { globals::tempAttributes["RoleName"] = globals::tempAttributes["name"]; }
     break;
 
   case 55:
-#line 365 "frontend-parser-wsdl.yy"
+#line 369 "frontend-parser-wsdl.yy"
     { temp_partnerLinkType->addRole(globals::tempAttributes["RoleName"], globals::tempAttributes["name"]); }
     break;
 
   case 57:
-#line 376 "frontend-parser-wsdl.yy"
+#line 380 "frontend-parser-wsdl.yy"
     { globals::tempAttributes[strip_namespace((yyvsp[(1) - (4)].yt_casestring)->name)] = strip_namespace((yyvsp[(3) - (4)].yt_casestring)->name); }
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 1916 "frontend-parser-wsdl.cc"
+#line 1924 "frontend-parser-wsdl.cc"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
