@@ -28,25 +28,32 @@
  * 
  * \since   2007/03/05
  *
- * \date    \$Date: 2007/05/10 12:07:17 $
+ * \date    \$Date: 2007/06/07 09:17:43 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.24 $
+ * \version \$Revision: 1.31 $
  */
 
 
 
 
 
+/******************************************************************************
+ * Headers
+ *****************************************************************************/
+
 #include "globals.h"
 #include "extension-wsdl.h"
 #include "extension-chor.h"
 #include "ast-details.h"
 
 
+
+
+
 namespace globals {
   string currentScope;
 
@@ -58,10 +65,13 @@ namespace globals {
   map<string, unsigned int> ASTE_outputChannels;
   set<string> ASTE_correlationSetNames;
   set<string> ASTE_variableNames;
+  set<string> ASTE_messageExchangeNames;
   set<string> ASTE_partnerLinkNames;
   set<string> ASTE_linkNames;
+  set<unsigned int> ASTE_IMAs;
   set<unsigned int> ASTE_startActivities;
 
+  map<string, string> ASTE_variableMap;
   map<string, unsigned int> ASTE_linkIdMap;
   map<string, unsigned int> ASTE_scopeNames;
   map<string, unsigned int> ASTE_partnerLinks;
@@ -104,6 +114,8 @@ namespace globals {
 
   bool parsing = true;
 
+  bool hideStandardHandlers = false;
+
   set<string> unknown_elements;
 
   bool abstract_process = false;
@@ -120,5 +132,7 @@ namespace globals {
 
   map<string, string> tempAttributes;
 
-  unsigned int instances_of_current_process = 0;
+  int instances_of_current_process = 0;
+  
+  map<unsigned int, unsigned int> if_branches;
 }
