@@ -28,13 +28,13 @@
  * 
  * \since   2007/03/05
  *
- * \date    \$Date: 2007/05/10 12:07:17 $
+ * \date    \$Date: 2007/06/07 09:17:43 $
  * 
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.24 $
+ * \version \$Revision: 1.33 $
  */
 
 
@@ -44,6 +44,14 @@
 #ifndef GLOBALS_H
 #define GLOBALS_H
 
+
+
+
+
+/******************************************************************************
+ * Headers
+ *****************************************************************************/
+
 #include <set>
 #include <string>
 #include <map>
@@ -63,6 +71,7 @@ class BPEL4Chor;
 
 
 
+
 /*!
  * \brief process information
  */
@@ -135,15 +144,24 @@ namespace globals
   /// set of (prefixed) variable names
   extern set<string> ASTE_variableNames;
 
+  /// set of (prefixed) messageExchange names
+  extern set<string> ASTE_messageExchangeNames;
+
   /// set of (prefixed) partner link names
   extern set<string> ASTE_partnerLinkNames;
 
   /// set of (prefixed) link names
   extern set<string> ASTE_linkNames;
 
+  /// set of incoming message activities
+  extern set<unsigned int> ASTE_IMAs;
+
   /// set of ASTE identifiers of the start activities
   extern set<unsigned int> ASTE_startActivities;
 
+  /// mapping of (prefixed) variable names to corresponding message types
+  extern map<string, string> ASTE_variableMap;
+
   /// mapping of link names to ASTE identifiers
   extern map<string, unsigned int> ASTE_linkIdMap;
 
@@ -166,9 +184,6 @@ namespace globals
   /// mapping of join conditions and an assignment number to a Boolean result.
   extern map<unsigned int, map<unsigned int, bool > > joinCondition_result;
 
-  /// \todo comment me
-  extern map<unsigned int, unsigned int> PPhasCompensateMap;
-
   /// the identifier of the current scope
   extern unsigned int PPcurrentScope;
 
@@ -227,6 +242,9 @@ namespace globals
   /// true while parsing, false while postprocessing
   extern bool parsing;
 
+  /// used to hide Handlers in Visualization
+  extern bool hideStandardHandlers;
+
   /// all elements that caused parse errors (either misplaced or nonstandard)
   extern set<string> unknown_elements;
 
@@ -246,7 +264,10 @@ namespace globals
   extern map<string, string> tempAttributes;
 
   /// the number of instances to be created of current process (used in BPEL4Chor)
-  extern unsigned int instances_of_current_process;
+  extern int instances_of_current_process;
+  
+  /// a mapping holding the number of branches for every <if> activity (indexed by the <if>'s id)
+  extern map<unsigned int, unsigned int> if_branches;
 }
 
 #endif
