@@ -29,13 +29,13 @@
  *
  * \since   2005/10/18
  *
- * \date    \$Date: 2007/04/29 19:09:57 $
+ * \date    \$Date: 2007/06/13 20:01:05 $
  *
  * \note    This file is part of the tool BPEL2oWFN and was created during the
  *          project "Tools4BPEL" at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.53 $
+ * \version \$Revision: 1.57 $
  */
 
 
@@ -89,7 +89,7 @@ typedef enum
    O_PARAMETER,		///< set various parameters (see #possibleParameters)
    O_DEBUG,		///< set the debug level
    O_REDUCTION,		///< set level of structural reduction
-   O_CHOR,		///< parse a choreography file (BPEL4Chor)
+   O_TOPOLOGY,		///< parse a choreography file (BPEL4Chor)
    O_WSDL		///< parse a WSDL file
 } possibleOptions;
 
@@ -103,11 +103,11 @@ typedef enum
 typedef enum
 {
   M_AST = 1,		///< abstract syntax tree
-  M_PRETTY,		///< BPEL pretty-printer
+  M_PRETTY,			///< BPEL pretty-printer
   M_PETRINET,		///< Petri net
-  M_CONSISTENCY,	///< consistency (parse two input files)
-  M_CFG,		///< control flow analysis
-  M_VIS 		///< BPEL process visualization
+  M_CHOREOGRAPHY,	///< consistency (parse two input files)
+  M_CFG,			///< control flow analysis
+  M_VIS				///< BPEL process visualization
 } possibleModi;
 
 
@@ -148,7 +148,9 @@ typedef enum
   P_XOR,		///< use XOR as transition condition
   P_LOOPCOUNT,		///< use loop counters
   P_LOOPCONTROL,	///< add communication to loops
-  P_NOINTERFACE		///< do not draw an interface
+  P_NOINTERFACE,	///< do not draw an interface
+  P_DEADLOCKTEST,	///< removes the deadlock from the final place
+  P_DATA		///< creates multiple nets (Diplomarbeit Thomas Heidinger)   
 } possibleParameters;
 
 
