@@ -30,13 +30,13 @@
  *
  * \since   created: 2006-03-16
  *
- * \date    \$Date: 2007/05/03 10:28:42 $
+ * \date    \$Date: 2007/06/05 15:04:00 $
  *
  * \note    This file is part of the tool GNU BPEL2oWFN and was created during
  *          the project Tools4BPEL at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.97 $
+ * \version \$Revision: 1.100 $
  *
  * \ingroup petrinet
  */
@@ -134,7 +134,7 @@ string Node::nodeShortName() const
  */
 string Node::nodeName() const
 {
-  string result = history[0];
+  string result = *history.begin();
 
   if ( type == INTERNAL || nodeType == TRANSITION )
   {
@@ -156,7 +156,7 @@ string Node::nodeName() const
  */
 string Node::nodeFullName() const
 {
-  string result = history[0];
+  string result = *history.begin();
 
   if ( type == INTERNAL || nodeType == TRANSITION )
     result = prefix + result;
@@ -176,13 +176,13 @@ string Node::nodeFullName() const
  */
 string Place::nodeShortName() const
 {
-  if (history[0].find("link") != string::npos || history[0].find("!link") != string::npos)
-    return history[0];
+  if (history.begin()->find("link") != string::npos || history.begin()->find("!link") != string::npos)
+    return *history.begin();
 
   if (type == INTERNAL)
     return ("p" + toString(id));
   
-  return history[0];
+  return *history.begin();
 }
 
 
@@ -234,59 +234,59 @@ string PetriNet::information() const
 void PetriNet::output_info(ostream *output) const
 {
   assert(output != NULL);
-
+  
   // the places
   (*output) << "PLACES:\nID\tTYPE\t\tROLES\n";
-
+  
   // the internal places
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
   {
     (*output) << (*p)->nodeShortName() << "\tinternal";
-
-    for (vector<string>::iterator role = (*p)->history.begin(); role != (*p)->history.end(); role++)
+    
+    for (list<string>::iterator role = (*p)->history.begin(); role != (*p)->history.end(); role++)
       if (role == (*p)->history.begin())
-	(*output) << "\t" + (*p)->prefix + *role + "\n";
+        (*output) << "\t" + (*p)->prefix + *role + "\n";
       else
-	(*output) << "\t\t\t" + (*p)->prefix + *role + "\n";
+        (*output) << "\t\t\t" + (*p)->prefix + *role + "\n";
   }
-
+  
   // the input places
   for (set<Place *>::iterator p = P_in.begin(); p != P_in.end(); p++)
   {
     (*output) << (*p)->nodeShortName() << "\tinput   ";
-
-    for (vector<string>::iterator role = (*p)->history.begin(); role != (*p)->history.end(); role++)
+    
+    for (list<string>::iterator role = (*p)->history.begin(); role != (*p)->history.end(); role++)
       if (role == (*p)->history.begin())
-	(*output) << "\t" + (*p)->prefix + *role + "\n";
+        (*output) << "\t" + (*p)->prefix + *role + "\n";
       else
-	(*output) << "\t\t\t" + (*p)->prefix + *role + "\n";
+        (*output) << "\t\t\t" + (*p)->prefix + *role + "\n";
   }
-
+  
   // the output places
   for (set<Place *>::iterator p = P_out.begin(); p != P_out.end(); p++)
   {
     (*output) << (*p)->nodeShortName() << "\toutput  ";
-
-    for (vector<string>::iterator role = (*p)->history.begin(); role != (*p)->history.end(); role++)
+    
+    for (list<string>::iterator role = (*p)->history.begin(); role != (*p)->history.end(); role++)
       if (role == (*p)->history.begin())
-	(*output) << "\t" + (*p)->prefix + *role + "\n";
+        (*output) << "\t" + (*p)->prefix + *role + "\n";
       else
-	(*output) << "\t\t\t" + (*p)->prefix + *role + "\n";
+        (*output) << "\t\t\t" + (*p)->prefix + *role + "\n";
   }
-
+  
   // the transitions
   (*output) << "\nTRANSITIONS:\n";
   (*output) << "ID\tROLES\n";
-
+  
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
   {
     (*output) << (*t)->nodeShortName() + "\t";
-
-    for (vector<string>::iterator role = (*t)->history.begin(); role != (*t)->history.end(); role++)
+    
+    for (list<string>::iterator role = (*t)->history.begin(); role != (*t)->history.end(); role++)
       if (role == (*t)->history.begin())
-	(*output) << (*t)->prefix + *role + "\n";
+        (*output) << (*t)->prefix + *role + "\n";
       else
-	(*output) << "\t" + (*t)->prefix + *role + "\n";
+        (*output) << "\t" + (*t)->prefix + *role + "\n";
   }
 }
 
@@ -306,9 +306,9 @@ string Arc::output_dot(bool draw_interface) const
 {
   if (!draw_interface)
     if ((source->nodeType == PLACE && source->type != INTERNAL) ||
-	(target->nodeType == PLACE && target->type != INTERNAL))
+        (target->nodeType == PLACE && target->type != INTERNAL))
       return "";
-
+  
   string result = " ";
   if (source->nodeType == PLACE)
     result += "p" + toString(source->id) + " -> t" + toString(target->id);
@@ -316,16 +316,16 @@ string Arc::output_dot(bool draw_interface) const
     result += "t" + toString(source->id) + " -> p" + toString(target->id);
   
   result += "\t[";
-
+  
   if (weight != 1)
     result += "label=\"" + toString(weight) + "\"";
-
-    if ((source->nodeType == PLACE && source->type == INTERNAL) ||
-	(target->nodeType == PLACE && target->type == INTERNAL))
-      result += "weight=10000.0";
-
+  
+  if ((source->nodeType == PLACE && source->type == INTERNAL) ||
+      (target->nodeType == PLACE && target->type == INTERNAL))
+    result += "weight=10000.0";
+  
   result += "]\n";
-
+  
   return result;
 }
 
@@ -344,104 +344,104 @@ string Arc::output_dot(bool draw_interface) const
 string Transition::output_dot() const
 {
   string result;
-
+  
   result += " t" + toString(id) + "  \t";
-
+  
 #ifdef USING_BPEL2OWFN
   string label = nodeShortName();
 #else
   string label = nodeName();
 #endif
-
+  
   switch(type)
   {
     case(IN):		result += "[fillcolor=orange "; break;
     case(OUT):		result += "[fillcolor=yellow "; break;
     case(INOUT):	result += "[fillcolor=gold ";
-                        result += "label=<";
-                        // the table size depends on the node size of .3 (inch);
-                        result += " <TABLE BORDER=\"1\"";
-                        result += " CELLBORDER=\"0\" CELLSPACING=\"0\" CELLPADDING=\"0\"";
-                        result += " HEIGHT=\"21\" WIDTH=\"21\" FIXEDSIZE=\"TRUE\"><TR>";
-                        result += "<TD HEIGHT=\"11\" WIDTH=\"21\" FIXEDSIZE=\"TRUE\" BGCOLOR=\"ORANGE\">";
-                        result += "</TD></TR><TR>";
-                        result += "<TD HEIGHT=\"10\" WIDTH=\"21\" FIXEDSIZE=\"TRUE\" BGCOLOR=\"YELLOW\">";
-                        result += "</TD></TR></TABLE> >"; break;
+      result += "label=<";
+      // the table size depends on the node size of .3 (inch);
+      result += " <TABLE BORDER=\"1\"";
+      result += " CELLBORDER=\"0\" CELLSPACING=\"0\" CELLPADDING=\"0\"";
+      result += " HEIGHT=\"21\" WIDTH=\"21\" FIXEDSIZE=\"TRUE\"><TR>";
+      result += "<TD HEIGHT=\"11\" WIDTH=\"21\" FIXEDSIZE=\"TRUE\" BGCOLOR=\"ORANGE\">";
+      result += "</TD></TR><TR>";
+      result += "<TD HEIGHT=\"10\" WIDTH=\"21\" FIXEDSIZE=\"TRUE\" BGCOLOR=\"YELLOW\">";
+      result += "</TD></TR></TABLE> >"; break;
     case(INTERNAL):	result += "["; break;
   }
-
-
+  
+  
   // add labels for transitions with singleton history
-
+  
   // internal activities
-  if (history.size() == 1 && history[0].find("internal.empty") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.empty") != string::npos)
     result += "label=\"empty\" fillcolor=gray";
-  if (history.size() == 1 && history[0].find("internal.assign") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.assign") != string::npos)
     result += "label=\"asgn\" fillcolor=gray";
-  if (history.size() == 1 && history[0].find("internal.opaqueActivity") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.opaqueActivity") != string::npos)
     result += "label=\"opque\" fillcolor=gray";
-
+  
   // communicating activities
-  if (history.size() == 1 && history[0].find("internal.receive") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.receive") != string::npos)
     result += "label=\"recv\"";
-  if (history.size() == 1 && history[0].find("internal.reply") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.reply") != string::npos)
     result += "label=\"reply\"";
-  if (history.size() == 1 && history[0].find("internal.invoke") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.invoke") != string::npos)
     result += "label=\"invk\"";
-  if (history.size() == 1 && history[0].find("internal.onMessage_") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.onMessage_") != string::npos)
     result += "label=\"on\\nmsg\"";
-  if (history.size() == 1 && history[0].find(".onEvent.") != string::npos)
+  if (history.size() == 1 && history.begin()->find(".onEvent.") != string::npos)
     result += "label=\"on\\nevent\"";
-
+  
   // structured activities
-  if (history.size() == 1 && history[0].find("internal.case") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.case") != string::npos)
     result += "label=\"case\" fillcolor=azure2";
-  if (history.size() == 1 && history[0].find("internal.onAlarm") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.onAlarm") != string::npos)
     result += "label=\"on\\nalarm\" fillcolor=azure2";
-  if (history.size() == 1 && history[0].find("internal.split") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.split") != string::npos)
     result += "label=\"flow\\nsplit\" fillcolor=azure2";
-  if (history.size() == 1 && history[0].find("internal.join") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.join") != string::npos)
     result += "label=\"flow\\njoin\" fillcolor=azure2";
-  if (history.size() == 1 && history[0].find("internal.leave") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.leave") != string::npos)
     result += "label=\"leave\\nloop\" fillcolor=azure2";
-  if (history.size() == 1 && history[0].find("internal.loop") != string::npos)
+  if (history.size() == 1 && history.begin()->find("internal.loop") != string::npos)
     result += "label=\"enter\\nloop\" fillcolor=azure2";
-
+  
   // everything about links
-  if (history.size() == 1 && history[0].find(".setLinks") != string::npos)
+  if (history.size() == 1 && history.begin()->find(".setLinks") != string::npos)
     result += "label=\"tc\" fillcolor=darkseagreen1";
-  if (history.size() == 1 && history[0].find(".evaluate") != string::npos)
+  if (history.size() == 1 && history.begin()->find(".evaluate") != string::npos)
     result += "label=\"jc\\neval\" fillcolor=darkseagreen1";
-  if (history.size() == 1 && history[0].find(".skip") != string::npos)
+  if (history.size() == 1 && history.begin()->find(".skip") != string::npos)
     result += "label=\"skip\" fillcolor=darkseagreen1";
-  if (history.size() == 1 && history[0].find(".reset_false") != string::npos)
+  if (history.size() == 1 && history.begin()->find(".reset_false") != string::npos)
     result += "label=\"reset\\nlink\" fillcolor=darkseagreen1";
-  if (history.size() == 1 && history[0].find(".reset_true") != string::npos)
+  if (history.size() == 1 && history.begin()->find(".reset_true") != string::npos)
     result += "label=\"reset\\nlink\" fillcolor=darkseagreen1";
   
-
+  
   // stopping
-  if (history.size() == 1 && history[0].find(".stopped.") != string::npos)
+  if (history.size() == 1 && history.begin()->find(".stopped.") != string::npos)
     result += "label=\"stop\" fillcolor=darksalmon";
   
-
+  
   if (!labels.empty())
   {
     result += " label=\"{";
     for (set<string>::const_iterator it = labels.begin(); it != labels.end(); it++)
     {
       if (it != labels.begin())
-	result += " ";
+        result += " ";
       result += (*it);
     }
     result += "}\"";
   }
   
   result += "]\n";
-
+  
   result += " t" + toString(id) + "_l\t[shape=none];\n";
   result += " t" + toString(id) + "_l -> t" + toString(id) + " [headlabel=\"" + label + "\"]\n";
-
+  
   return result;
 }
 
@@ -460,35 +460,50 @@ string Transition::output_dot() const
 string Place::output_dot() const
 {
   string result;
-
+  
   result += " p" + toString(id) + "  \t[";//label=\"\"";
-
+    
 #ifdef USING_BPEL2OWFN
-  string label;
-  if ( wasExternal != "")
-    label = wasExternal;
+    string label;
+    if ( wasExternal != "")
+      label = wasExternal;
   else
     label = nodeShortName();
+  
+  if (type == IN || type == OUT)
+  {
+    // strip "in." or "out."
+    label = label.substr(label.find_first_of(".")+1, label.length());
+    
+    // if channel is instantiated, strip the ".instance_" part and replace 
+    if (label.find(".instance_") != string::npos)
+    {
+      string label_prefix = label.substr(0, label.find(".instance_"));
+      string label_suffix = label.substr(label.find(".instance_")+10, label.length());
+      label = label_prefix + " (" + label_suffix + ")";
+    }
+  }
+  
 #else
   string label = nodeName();
 #endif
-
+  
   // truncate prefix (could be a problem with ports later on, but looks nice)
-//  if (type != INTERNAL)
-    // label = label.substr(label.find_last_of(".")+1, label.length());
+  //  if (type != INTERNAL)
+  // label = label.substr(label.find_last_of(".")+1, label.length());
   
   if (tokens == 1)
     result += "fillcolor=black peripheries=2 height=\".2\" width=\".2\" ";
   else if (tokens > 1)
     result += "label=\"" + toString(tokens) + "\" fontcolor=black fontname=\"Helvetica\" fontsize=10";
-
+  
   switch (type)
   {
     case (IN):  result += "fillcolor=orange"; break;
     case (OUT): result += "fillcolor=yellow"; break;
     default:    break;
   }
-
+  
   if (isFinal)
     result += "fillcolor=gray";
   else if (firstMemberIs("!link."))
@@ -501,16 +516,16 @@ string Place::output_dot() const
     result += "fillcolor=seagreen";
   else if (wasExternal != "")
     result += "fillcolor=lightgoldenrod1";
-    
+  
   result += "]\n";
-
+  
   result += " p" + toString(id) + "_l\t[shape=none];\n";
-
+  
   if (type == OUT)
     result += " p" + toString(id) + " -> p" + toString(id) + "_l [taillabel=\"" + label + "\"]\n";
   else
     result += " p" + toString(id) + "_l -> p" + toString(id) + " [headlabel=\"" + label + "\"]\n";
-
+  
   return result;
 }
 
@@ -533,26 +548,26 @@ string Place::output_dot() const
 void PetriNet::output_dot(ostream *output, bool draw_interface) const
 {
   assert(output != NULL);
-
+  
   (*output) << "digraph N {" << endl;
   (*output) << " graph [fontname=\"Helvetica\" nodesep=0.3 ranksep=\"0.2 equally\" fontsize=10 label=\"";
-
+  
   if (globals::reduction_level == 5)
     (*output) << "structurally reduced ";
-
+  
   (*output) << "Petri net generated from " << globals::filename << "\"]" << endl;
   // REMEMBER The table size of the INOUT transitions depends on the size of a node!
   //          So a width of .3 (in) results in 21 pixel table width ( 0.3 in * 72 dpi ).
   (*output) << " node [fontname=\"Helvetica\" fontsize=8 fixedsize width=\".3\" height=\".3\" label=\"\" style=filled fillcolor=white]" << endl;
   (*output) << " edge [fontname=\"Helvetica\" fontsize=8 color=white arrowhead=none weight=\"20.0\"]" << endl << endl;
-
-
+  
+  
   // list the places
   (*output) << "\n // places" << endl;
   (*output) << " node [shape=circle];" << endl;
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
     (*output) << (*p)->output_dot();
-
+  
   if (draw_interface)
   {
     for (set<Place *>::iterator p = P_in.begin(); p != P_in.end(); p++)
@@ -560,14 +575,14 @@ void PetriNet::output_dot(ostream *output, bool draw_interface) const
     for (set<Place *>::iterator p = P_out.begin(); p != P_out.end(); p++)
       (*output) << (*p)->output_dot();
   }
-
+  
   // list the transitions
   (*output) << "\n // transitions" << endl;
   (*output) << " node [shape=box]" << endl;
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
     (*output) << (*t)->output_dot();
-
-
+  
+  
   (*output) << "\n // cluster the inner of the net" << endl;
   (*output) << " subgraph cluster1\n {\n ";
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
@@ -580,21 +595,21 @@ void PetriNet::output_dot(ostream *output, bool draw_interface) const
     else
       (*output) << " p" << (*p)->id << " p" << (*p)->id << "_l";
   }
-
+  
   if (draw_interface)
     (*output) << "\n  label=\"\" style=dashed" << endl;
   else
     (*output) << "\n  label=\"\" style=invis" << endl;
-
+  
   (*output) << " }" << endl;
-
-
+  
+  
   // list the arcs
   (*output) << "\n // arcs" << endl;
   (*output) << " edge [fontname=\"Helvetica\" fontsize=8 arrowhead=normal color=black]" << endl;
   for (set<Arc *>::iterator f = F.begin(); f != F.end(); f++)
     (*output) << (*f)->output_dot(draw_interface);
-
+  
   (*output) << "}" << endl;
 }
 
@@ -622,17 +637,17 @@ void PetriNet::output_dot(ostream *output, bool draw_interface) const
 void PetriNet::output_pnml(ostream *output) const
 {
   assert(output != NULL);
-
+  
   (*output) << "<?xml version=\"1.0\" encoding=\"utf-8\"?>" << endl << endl;
   (*output) << "<!--" << endl;
   (*output) << "  Petri net created by " << PACKAGE_STRING << " reading file " << globals::filename << "." << endl;
   (*output) << "  See http://www.gnu.org/software/bpel2owfn for more details." << endl;
   (*output) << "-->" << endl << endl;
-
+  
   (*output) << "<pnml>" << endl;
   (*output) << "  <net id=\"bpel-net\" type=\"owfn\">" << endl << endl;
-
-
+  
+  
   // places
   (*output) << "<!-- input places -->" << endl;
   for (set<Place *>::iterator p = P_in.begin(); p != P_in.end(); p++)
@@ -644,14 +659,14 @@ void PetriNet::output_pnml(ostream *output) const
     (*output) << "    <place id=\"" << (*p)->nodeName() << "\">" << endl;
 #endif
     (*output) << "      <name>" << endl;
-    (*output) << "        <text>" << (*p)->history[0] << "</text>" << endl;
+    (*output) << "        <text>" << *(*p)->history.begin() << "</text>" << endl;
     (*output) << "      </name>" << endl;
     (*output) << "      <type>" << endl;
     (*output) << "        <text>input</text>" << endl;
     (*output) << "      </type>" << endl;
     (*output) << "    </place>" << endl << endl;
   }
-
+  
   (*output) << "<!-- output places -->" << endl;
   for (set<Place *>::iterator p = P_out.begin(); p != P_out.end(); p++)
   {
@@ -662,14 +677,14 @@ void PetriNet::output_pnml(ostream *output) const
     (*output) << "    <place id=\"" << (*p)->nodeName() << "\">" << endl;
 #endif
     (*output) << "      <name>" << endl;
-    (*output) << "        <text>" << (*p)->history[0] << "</text>" << endl;
+    (*output) << "        <text>" << *(*p)->history.begin() << "</text>" << endl;
     (*output) << "      </name>" << endl;
     (*output) << "      <type>" << endl;
     (*output) << "        <text>output</text>" << endl;
     (*output) << "      </type>" << endl;
     (*output) << "    </place>" << endl << endl;
   }
-
+  
   (*output) << "<!-- internal places -->" << endl;
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
   {
@@ -680,7 +695,7 @@ void PetriNet::output_pnml(ostream *output) const
     (*output) << "    <place id=\"" << (*p)->nodeName() << "\">" << endl;
 #endif
     (*output) << "      <name>" << endl;
-    (*output) << "        <text>" << (*p)->history[0] << "</text>" << endl;
+    (*output) << "        <text>" << *(*p)->history.begin() << "</text>" << endl;
     (*output) << "      </name>" << endl;
     if ((*p)->tokens > 0)
     {
@@ -690,8 +705,8 @@ void PetriNet::output_pnml(ostream *output) const
     }
     (*output) << "    </place>" << endl << endl;
   }
-
-
+  
+  
   // transitions
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
   {
@@ -702,12 +717,12 @@ void PetriNet::output_pnml(ostream *output) const
     (*output) << "    <transition id=\"" << (*t)->nodeName() << "\">" << endl;
 #endif
     (*output) << "      <name>" << endl;
-    (*output) << "        <text>" << (*t)->history[0] << "</text>" << endl;
+    (*output) << "        <text>" << *(*t)->history.begin() << "</text>" << endl;
     (*output) << "      </name>" << endl;
     (*output) << "    </transition>" << endl << endl;
   }
   (*output) << endl;
-
+  
   // arcs
   int arcNumber = 1;
   for (set<Arc *>::iterator f = F.begin(); f != F.end(); f++, arcNumber++)
@@ -725,15 +740,15 @@ void PetriNet::output_pnml(ostream *output) const
     (*output) << "    </arc>" << endl;
   }
   (*output) << endl;
-
+  
   (*output) << "    <toolspecific tool=\"owfn\" version=\"1.0\">" << endl;
   (*output) << "      <finalmarking xmlns=\"http://www.informatik.hu-berlin.de/top/tools4bpel\">" << endl;
   (*output) << "        <text>";
-
+  
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
     if ((*p)->isFinal)
       (*output) << (*p)->nodeShortName() << " ";
-
+  
   (*output) << "</text>" << endl;
   (*output) << "      </finalmarking>" << endl;
   (*output) << "    </toolspecific>" << endl;
@@ -761,10 +776,10 @@ void PetriNet::output_pnml(ostream *output) const
 void PetriNet::output_pep(ostream *output) const
 {
   assert(output != NULL);
-
+  
   // header
   (*output) << "PEP" << endl << "PTNet" << endl << "FORMAT_N" << endl;
-
+  
   // places (only internal)
   (*output) << "PL" << endl;
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
@@ -774,25 +789,25 @@ void PetriNet::output_pep(ostream *output) const
       (*output) << "M" << (*p)->tokens;
     (*output) << "k1" << endl;
   }
-
+  
   // transitions
   (*output) << "TR" << endl;
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
     (*output) << (*t)->id << "\"" << (*t)->nodeShortName() << "\"80@40" << endl;
-
+  
   // arcs from transitions to places (no output places)
   (*output) << "TP" << endl;
   for (set<Arc *>::iterator f = F.begin(); f != F.end(); f++)
     if (((*f)->source->nodeType) == TRANSITION)
       if ( P_out.find(static_cast<Place*>((*f)->target)) == P_out.end())
-	(*output) << (*f)->source->id << "<" << (*f)->target->id << "w" << (*f)->weight << endl;
-
+        (*output) << (*f)->source->id << "<" << (*f)->target->id << "w" << (*f)->weight << endl;
+  
   // arcs from places to transitions
   (*output) << "PT" << endl;
   for (set<Arc *>::iterator f = F.begin(); f != F.end(); f++)
     if (((*f)->source->nodeType) == PLACE)
       if ( P_in.find(static_cast<Place*>((*f)->source)) == P_in.end())
-	(*output) << (*f)->source->id << ">" << (*f)->target->id << "w" << (*f)->weight << endl;
+        (*output) << (*f)->source->id << ">" << (*f)->target->id << "w" << (*f)->weight << endl;
 }
 
 
@@ -816,64 +831,64 @@ void PetriNet::output_pep(ostream *output) const
 void PetriNet::output_ina(ostream *output) const
 {
   assert(output != NULL);
-
+  
   // net header
   (*output) << "P   M   PRE,POST  NETZ 1:" << globals::filename.substr(0, globals::filename.find_first_of(".")) << endl;
-
+  
   // places (only internal)
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
   {
     (*output) << setw(3) << (*p)->id << " " << (*p)->tokens << "     ";
-
+    
     for (set<Node*>::iterator t = (*p)->preset.begin(); t != (*p)->preset.end(); t++)
     {
       if (t != (*p)->preset.begin())
-	(*output) << " ";
+        (*output) << " ";
       
       (*output) << (*t)->id << ":" << arc_weight(*t, *p);
     }
-
+    
     (*output) << ", ";
-
+    
     for (set<Node*>::iterator t = (*p)->postset.begin(); t != (*p)->postset.end(); t++)
     {
       if (t != (*p)->postset.begin())
-	(*output) << " ";
+        (*output) << " ";
       
       (*output) << (*t)->id << ":" << arc_weight(*p, *t);
     }
-
+    
     (*output) << endl;
   }
-
+  
   (*output) << "@" << endl;
-
-
+  
+  
   // place names
   (*output) << "place nr.             name capacity time" << endl;
-
+  
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
   {
     (*output) << right << setw(8) << (*p)->id << ": ";
     (*output) << left << setw(16) << (*p)->nodeShortName();
     (*output) << "       oo    0" << endl;
   }
-
+  
   (*output) << "@" << endl;
-
+  
   
   // transition names
   (*output) << "trans nr.             name priority time" << endl;
-
+  
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
   {
     (*output) << right << setw(8) << (*t)->id << ": ";
     (*output) << left << setw(16) << (*t)->nodeShortName();
     (*output) << "        0    0" << endl;
   }
-
+  
   (*output) << "@" << endl;
-
+  
   // info (ignored after last '@')
   (*output) << "generated by: " << PACKAGE_STRING << endl;
   (*output) << "input file:   `" << globals::filename << "'" << endl;
@@ -900,17 +915,17 @@ void PetriNet::output_spin(ostream *output) const
   assert(output != NULL);
   // net header
   (*output) << "/* NO 1-safe */" << endl;
-
+  
   // places (only internal)
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
   {
     (*output) << "byte p" << (*p)->id << "=" << (*p)->tokens << ";" << endl;
   }
-
+  
   (*output) << "int {" << endl;
   (*output) << "\tdo" << endl;
-
-
+  
+  
   // transitions
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
   {
@@ -936,27 +951,27 @@ void PetriNet::output_spin(ostream *output) const
         (*output) << "(p" << (*p)->id << ">=" << arc_weight(*p,*t) << ")";
       }
     }
-
+    
     (*output) << ") -> ";
-  
+    
     for (set<Node *>::iterator p = (*t)->preset.begin(); p != (*t)->preset.end(); p++)
     {
       (*output) << "p" << (*p)->id << "=p" << (*p)->id << "-" << arc_weight(*p,*t) << ";";
     }
-
+    
     (*output) << endl << "\t\t\t";
     
     for (set<Node *>::iterator p = (*t)->postset.begin(); p != (*t)->postset.end(); p++)
     {
       (*output) << "p" << (*p)->id << "=p" << (*p)->id << "+" << arc_weight(*t,*p) << ";";
     }
-
+    
     (*output) << " }" << endl;
     
   }
   
-    (*output) << "\tod;" << endl;
-    (*output) << "}" << endl;
+  (*output) << "\tod;" << endl;
+  (*output) << "}" << endl;
 }
 
 
@@ -977,9 +992,9 @@ void PetriNet::output_spin(ostream *output) const
 void PetriNet::output_apnn(ostream *output) const
 {
   assert(output != NULL);
-
+  
   (*output) << "\\beginnet{" << globals::filename << "}" << endl << endl;
-
+  
   // places (only internal)
   for (set<Place *>::iterator p = P.begin(); p != P.end(); p++)
   {
@@ -989,14 +1004,14 @@ void PetriNet::output_apnn(ostream *output) const
     (*output) << "}" << endl;
   }
   (*output) << endl;
-
+  
   // transitions
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
   {
     (*output) << "  \\transition{" << (*t)->nodeShortName() << "}{}" << endl;
   }
   (*output) << endl;
-
+  
   // arcs
   int arcNumber = 1;
   for (set<Arc *>::iterator f = F.begin(); f != F.end(); f++, arcNumber++)
@@ -1004,19 +1019,19 @@ void PetriNet::output_apnn(ostream *output) const
     // ignore input places
     if ((*f)->source->nodeType == PLACE)
       if ( P_in.find(static_cast<Place*>((*f)->source)) != P_in.end())
-	continue;
-
+        continue;
+    
     // ignore output places
     if ((*f)->target->nodeType == PLACE)
       if ( P_out.find(static_cast<Place*>((*f)->target)) != P_out.end())
-	continue;
-
+        continue;
+    
     (*output) << "  \\arc{a" << arcNumber << "}{ ";
     (*output) << "\\from{" << (*f)->source->nodeShortName() << " } ";
     (*output) << "\\to{" << (*f)->target->nodeShortName() << "} \\weight{" << (*f)->weight << "} }" << endl;
   }
   (*output) << endl;
-
+  
   (*output) << "\\endnet" << endl;
 }
 
@@ -1038,9 +1053,9 @@ void PetriNet::output_apnn(ostream *output) const
 void PetriNet::output_lola(ostream *output) const
 {
   assert(output != NULL);
-
+  
   (*output) << "{ Petri net created by " << PACKAGE_STRING << " reading " << globals::filename << " }" << endl << endl;
-
+  
   
   // places (only internal)
   (*output) << "PLACE" << endl;
@@ -1053,8 +1068,8 @@ void PetriNet::output_lola(ostream *output) const
       (*output) << "," << endl;
   }
   (*output) << endl << ";" << endl << endl << endl;
-
-
+  
+  
   // initial marking
   (*output) << "MARKING" << endl;
   count = 1;
@@ -1063,14 +1078,14 @@ void PetriNet::output_lola(ostream *output) const
     if ((*p)->tokens > 0)
     {
       if (count++ != 1)
-	(*output) << "," << endl;
+        (*output) << "," << endl;
       
       (*output) << "  " << (*p)->nodeShortName() << ":\t" << (*p)->tokens;
     }
   }
   (*output) << endl << ";" << endl << endl << endl;
-
-
+  
+  
   // transitions
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
   {
@@ -1082,13 +1097,13 @@ void PetriNet::output_lola(ostream *output) const
     {
       // ignore input places
       if ( (*pre)->nodeType == PLACE )
-	if ( P_in.find(static_cast<Place*>(*pre)) != P_in.end())
-	  continue;
-
+        if ( P_in.find(static_cast<Place*>(*pre)) != P_in.end())
+          continue;
+      
       (*output) << "  " << (*pre)->nodeShortName() << ":\t" << arc_weight(*pre, *t);
-
+      
       if (count < (*t)->preset.size())
-	(*output) << "," << endl;
+        (*output) << "," << endl;
     }
     (*output) << ";" << endl;
     
@@ -1098,13 +1113,13 @@ void PetriNet::output_lola(ostream *output) const
     {
       // ignore output places
       if ( (*post)->nodeType == PLACE )
-	if ( P_out.find(static_cast<Place*>(*post)) != P_out.end())
-	  continue;
-
+        if ( P_out.find(static_cast<Place*>(*post)) != P_out.end())
+          continue;
+      
       (*output) << "  " << (*post)->nodeShortName() << ":\t" << arc_weight(*t, *post);
-
+      
       if (count < (*t)->postset.size())
-	(*output) << "," << endl;
+        (*output) << "," << endl;
     }
     
     (*output) << ";" << endl << endl;
@@ -1135,7 +1150,7 @@ void PetriNet::output_lola(ostream *output) const
 void PetriNet::output_owfn(ostream *output) const
 {
   assert(output != NULL);
-
+  
   (*output) << "{" << endl;
   (*output) << "  generated by: " << PACKAGE_STRING << endl;
 #ifdef USING_BPEL2OWFN
@@ -1147,14 +1162,14 @@ void PetriNet::output_owfn(ostream *output) const
   (*output) << "  invocation:   `" << globals::invocation << "'" << endl;
   (*output) << "  net size:     " << information() << endl;
   (*output) << "}" << endl << endl;
-
+  
   // places
   (*output) << "PLACE" << endl;
-
+  
   // internal places
   (*output) << "  INTERNAL" << endl;
   (*output) << "    ";
-
+  
   unsigned int count = 1;
   for (set<Place *>::iterator p = P.begin(); p != P.end(); count++, p++)
   {
@@ -1167,8 +1182,8 @@ void PetriNet::output_owfn(ostream *output) const
       (*output) << ", ";
   }
   (*output) << ";" << endl << endl;
-
-
+  
+  
   // input places
   (*output) << "  INPUT" << endl;
   count = 1;
@@ -1179,18 +1194,18 @@ void PetriNet::output_owfn(ostream *output) const
 #else
     (*output) << "    " << (*p)->nodeName();
 #endif
-   
+    
     if ((*p)->max_occurrences != UINT_MAX && (*p)->max_occurrences != 0)
       (*output) << " {$ MAX_OCCURRENCES = " << (*p)->max_occurrences << " $}";
     if ((*p)->max_occurrences == UINT_MAX)
       (*output) << " {$ MAX_OCCURRENCES = -1 $}";
-
+    
     if (count < P_in.size())
       (*output) << "," << endl;
   }
   (*output) << ";" << endl << endl;
-
-
+  
+  
   // output places
   (*output) << "  OUTPUT" << endl;
   count = 1;
@@ -1206,13 +1221,13 @@ void PetriNet::output_owfn(ostream *output) const
       (*output) << " {$ MAX_OCCURRENCES = " << (*p)->max_occurrences << " $}";
     if ((*p)->max_occurrences == UINT_MAX)
       (*output) << " {$ MAX_OCCURRENCES = -1 $}";
-
+    
     if (count < P_out.size())
       (*output) << "," << endl;
   }
   (*output) << ";" << endl << endl << endl;
   
-
+  
   // initial marking
   (*output) << "INITIALMARKING" << endl;
   count = 1;
@@ -1221,21 +1236,21 @@ void PetriNet::output_owfn(ostream *output) const
     if ((*p)->tokens > 0)
     {
       if (count++ != 1)
-	(*output) << "," << endl;
-
+        (*output) << "," << endl;
+      
 #ifdef USING_BPEL2OWFN
       (*output) << "  " << (*p)->nodeShortName() << ":\t" << (*p)->tokens;
 #else
       (*output) << "  " << (*p)->nodeName() << ":\t" << (*p)->tokens;
 #endif
-
+      
       if ((*p)->historyContains("1.internal.initial"))
-	(*output) << " {initial place}";
+        (*output) << " {initial place}";
     }
   }
   (*output) << ";" << endl << endl;  
-
-
+  
+  
   // final marking
   (*output) << "FINALMARKING" << endl;
   count = 1;
@@ -1244,23 +1259,23 @@ void PetriNet::output_owfn(ostream *output) const
     if ( (*p)->isFinal )
     {
       if (count != 1)
-	(*output) << ",";
-
+        (*output) << ",";
+      
 #ifdef USING_BPEL2OWFN
       (*output) << "  " << (*p)->nodeShortName();
 #else
       (*output) << "  " << (*p)->nodeName();
 #endif
-
+      
       (*output) << " {final place}";
-
+      
       if (count++ != 1)
-	(*output) << endl;      
+        (*output) << endl;      
     }
   }
   (*output) << ";" << endl << endl << endl;
-
-
+  
+  
   // transitions
   for (set<Transition *>::iterator t = T.begin(); t != T.end(); t++)
   {
@@ -1276,7 +1291,7 @@ void PetriNet::output_owfn(ostream *output) const
       case(OUT):	(*output) << " { output }" << endl; break;
       case(INOUT):	(*output) << " { input/output }" << endl; break;
     }
-
+    
     (*output) << "  CONSUME ";
     count = 1;
     for (set<Node *>::iterator pre = (*t)->preset.begin(); pre != (*t)->preset.end(); count++, pre++)
@@ -1286,17 +1301,17 @@ void PetriNet::output_owfn(ostream *output) const
 #else
       (*output) << (*pre)->nodeName();
 #endif
-
+      
       if (arc_weight(*pre, *t) != 1)
-	(*output) << ":" << arc_weight(*pre, *t);
-
+        (*output) << ":" << arc_weight(*pre, *t);
+      
       if (count < (*t)->preset.size())
-	(*output) << ", ";
+        (*output) << ", ";
     }
     (*output) << ";" << endl;
     
     (*output) << "  PRODUCE ";
-
+    
     count = 1;
     for (set<Node *>::iterator post = (*t)->postset.begin(); post != (*t)->postset.end(); count++, post++)
     {
@@ -1306,10 +1321,10 @@ void PetriNet::output_owfn(ostream *output) const
       (*output) << (*post)->nodeName();
 #endif
       if (arc_weight(*t, *post) != 1)
-	(*output) << ":" << arc_weight(*t, *post);
+        (*output) << ":" << arc_weight(*t, *post);
       
       if (count < (*t)->postset.size())
-	(*output) << ", ";
+        (*output) << ", ";
     }
     
     (*output) << ";" << endl << endl;
@@ -1345,7 +1360,7 @@ ostream& operator<< (ostream& os, const PetriNet &obj)
     case(FORMAT_PNML):	obj.output_pnml(&os); break;
     default:		break;
   }
-
+  
   return os;
 }
 
