@@ -27,17 +27,17 @@
  * \author  Niels Lohmann <nlohmann@informatik.hu-berlin.de>,
  *          Christian Gierds <gierds@informatik.hu-berlin.de>,
  *          Martin Znamirowski <znamirow@informatik.hu-berlin.de>,
- *          last changes of: \$Author: gierds $
+ *          last changes of: \$Author: nielslohmann $
  *
  * \since   2005-10-18
  *
- * \date    \$Date: 2007/05/10 12:23:16 $
+ * \date    \$Date: 2007/06/05 15:04:00 $
  *
  * \note    This file is part of the tool GNU BPEL2oWFN and was created during
  *          the project Tools4BPEL at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.205 $
+ * \version \$Revision: 1.218 $
  *
  * \ingroup petrinet
  */
@@ -60,6 +60,7 @@
 
 using std::pair;
 using std::cerr;
+using std::endl;
 
 
 
@@ -109,10 +110,13 @@ bool Node::firstMemberIs(string role) const
  */
 bool Node::historyContains(string role) const
 {
-  for (unsigned int i = 0; i < history.size(); i++)
-    if (history[i] == role)
+  for (list<string>::const_iterator iter = history.begin(); iter != history.end(); iter++)
+  {
+    if ( *iter == role )
+    {
       return true;
-
+    }
+  }
   return false;
 }
 
@@ -146,6 +150,8 @@ Transition::Transition(unsigned int my_id, string my_role)
   nodeType = TRANSITION;
   type = INTERNAL;
 
+  max_occurrences = 0;
+
   if (my_role != "")
     history.push_back(my_role);
 }
@@ -191,13 +197,14 @@ void Transition::add_label(string new_label)
  */
 Place::Place(unsigned int my_id, string my_role, communication_type my_type) :
   tokens(0),
-  max_occurrences(0),
   isFinal(false)
 {
   id = my_id;
   type = my_type;
   nodeType = PLACE;
 
+  max_occurrences = 0;
+
   if (my_role != "")
     history.push_back(my_role);
 }
@@ -361,7 +368,7 @@ PetriNet::PetriNet(const PetriNet & net)
 
     roleMap[ newPlace->nodeFullName() ] = newPlace;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[*name] = newPlace;
   }
   // add all input places
@@ -373,7 +380,7 @@ PetriNet::PetriNet(const PetriNet & net)
 
     roleMap[ newPlace->nodeFullName() ] = newPlace;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[*name] = newPlace;
   }
   // add all output places
@@ -385,7 +392,7 @@ PetriNet::PetriNet(const PetriNet & net)
 
     roleMap[ newPlace->nodeFullName() ] = newPlace;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[*name] = newPlace;
   }
   // add all transitions
@@ -397,7 +404,7 @@ PetriNet::PetriNet(const PetriNet & net)
 
     roleMap[ newTransition->nodeFullName() ] = newTransition;
 
-    for(vector< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
+    for(list< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
       roleMap[ *name ] = newTransition;
   }
   // create arcs according to the given "net"
@@ -463,7 +470,7 @@ PetriNet & PetriNet::operator=(const PetriNet & net)
 
     roleMap[ newPlace->nodeFullName() ] = newPlace;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[*name] = newPlace;
   }
   // add all input places
@@ -477,7 +484,7 @@ PetriNet & PetriNet::operator=(const PetriNet & net)
 
     roleMap[ newPlace->nodeFullName() ] = newPlace;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[*name] = newPlace;
   }
   // add all output places
@@ -491,7 +498,7 @@ PetriNet & PetriNet::operator=(const PetriNet & net)
 
     roleMap[ newPlace->nodeFullName() ] = newPlace;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[*name] = newPlace;
   }
   // add all transitions
@@ -505,7 +512,7 @@ PetriNet & PetriNet::operator=(const PetriNet & net)
 
     roleMap[ newTransition->nodeFullName() ] = newTransition;
 
-    for(vector< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
+    for(list< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
       roleMap[ *name ] = newTransition;
   }
   // create arcs according to the given "net"
@@ -679,6 +686,7 @@ Arc *PetriNet::newArc(Node *my_source, Node *my_target, arc_type my_type, unsign
   assert(f != NULL);
   F.insert(f);
 
+  weight[ pair<Node*, Node*>(my_source, my_target) ] = my_weight;
 
   // Add a second arc to close a loop if the arc is a read arc.
   if (my_type == READ)
@@ -686,6 +694,7 @@ Arc *PetriNet::newArc(Node *my_source, Node *my_target, arc_type my_type, unsign
     Arc *f2 = new Arc(my_target, my_source, my_weight);
     assert(f2 != NULL);
     F.insert(f2);
+    weight[ pair<Node*, Node*>(my_target, my_source) ] = my_weight;
   }
 
   return f;
@@ -741,7 +750,7 @@ void PetriNet::removePlace(Place *p)
 
   // Remove the roles of the place p, i.e. set the mappings to the NULL
   // pointer.
-  for (vector<string>::iterator role = p->history.begin(); role != p->history.end(); role++)
+  for (list<string>::iterator role = p->history.begin(); role != p->history.end(); role++)
     if (roleMap[*role] == p)
       roleMap[*role] = NULL;
 
@@ -777,7 +786,7 @@ void PetriNet::removeTransition(Transition *t)
 
   // Remove the roles of the transition  t, i.e. set the mappings to the NULL
   // pointer.
-  for (vector<string>::iterator role = t->history.begin(); role != t->history.end(); role++)
+  for (list<string>::iterator role = t->history.begin(); role != t->history.end(); role++)
     if (roleMap[*role] == t)
       roleMap[*role] = NULL;
 
@@ -801,7 +810,10 @@ void PetriNet::removeArc(Arc *f)
   if (f == NULL)
     return;
 
+  weight[ pair<Node*, Node*>(f->source, f->target) ] = 0;
+
   F.erase(f);
+
   delete f;
 }
 
@@ -837,79 +849,105 @@ void PetriNet::mergeTransitions(Transition *t1, Transition *t2)
 {
   if (t1 == t2)
     return;
-
+  
   assert(t1 != NULL);
   assert(t2 != NULL);
   
+  // variable needed for correct arc weights
   bool sametarget = false;
+  // container for arcs to be deleted
   set<Arc *>::iterator delArc;
-  
+  // the merged transition
   Node *t12 = newTransition("");
-
+  
   // organize the communication type of the new transition
   if (t1->type == t2->type)
   {
     t12->type = t1->type;
   }
   else if ((t1->type == IN && t2->type == INTERNAL) ||
-      (t1->type == INTERNAL && t2->type == IN))
+           (t1->type == INTERNAL && t2->type == IN))
   {
     t12->type = IN;
   }
   else if ((t1->type == INTERNAL && t2->type == OUT) ||
-      (t1->type == OUT && t2->type == INTERNAL))
+           (t1->type == OUT && t2->type == INTERNAL))
   {
     t12->type = OUT;
   }
   else if ((t1->type == OUT && t2->type == IN) ||
-      (t1->type == IN && t2->type == OUT) ||
-      (t1->type == INOUT || t2->type == INOUT))
+           (t1->type == IN && t2->type == OUT) ||
+           (t1->type == INOUT || t2->type == INOUT))
   {
     t12->type = INOUT;
   }
   else assert(false); ///< this should never happer or we have missed a case
-
+  
+  t12->prefix = t1->prefix;
+  
   // copy t1's history to t12
-  for (vector<string>::iterator role = t1->history.begin(); role != t1->history.end(); role++)
+  for (list<string>::iterator role = t1->history.begin(); role != t1->history.end(); role++)
   {
     roleMap[*role] = t12;
     t12->history.push_back(*role);
+    //    if (t1->prefix != "")
+    {
+      //      roleMap[t1->prefix + *role] = t12;
+      //      t12->history.push_back(t1->prefix + *role);
+    }
   }
-
+  
   // copy t2's history to t12
-  for (vector<string>::iterator role = t2->history.begin(); role != t2->history.end(); role++)
+  for (list<string>::iterator role = t2->history.begin(); role != t2->history.end(); role++)
   {
     roleMap[*role] = t12;
     t12->history.push_back(*role);
+    //    if (t1->prefix != "" || t2->prefix != "")
+    {
+      //      roleMap[t2->prefix + *role] = t12;
+      //      t12->history.push_back(t2->prefix + *role);
+    }
   }
-
+  
+  roleMap[t1->nodeFullName()] = t12;
+  roleMap[t2->nodeFullName()] = t12;
+  
   // merge pre- and postsets for t12
   t12->preset=setUnion(t1->preset, t2->preset);
   t12->postset=setUnion(t1->postset, t2->postset);
-
+  
   // create the weighted arcs for t12
+  
+  // this is the preset of t12 without the preset of t1. It is needed not generate double arcs if
+  // the preset of t1 and t2 were not distinct.
   set<Node *> preset2without1 = setDifference(t12->preset,t1->preset);
-
+  
+  // create new arcs from the t1 preset to t12
   for (set<Node *>::iterator n = t1->preset.begin(); n != t1->preset.end(); n++)
     newArc((*n), t12, STANDARD, arc_weight((*n),t1));
-
+  
+  // create new arcs from the preset of t2 to t12 without those, that have been already covered by the preset of t1
   for (set<Node *>::iterator n = preset2without1.begin(); n != preset2without1.end(); n++)
     newArc((*n), t12, STANDARD, arc_weight((*n),t2));
-
+  
+  // create new arcs from t12 to postset of t1
   for (set<Node *>::iterator n = t1->postset.begin(); n != t1->postset.end(); n++) 
     newArc(t12, (*n), STANDARD, arc_weight(t1,(*n)));
-
+  
+  // create new arcs from t12 to postset of t2 and adjust arcweights if a node of postset of t2 is also in postset of t1
   for (set<Node *>::iterator n = t2->postset.begin(); n != t2->postset.end(); n++)
   {    
+    // Find an arc already created in because of the postset of t1
     for (set<Arc *>::iterator f = F.begin(); f != F.end(); f++)
     {
       if (((*f)->source == t12) && ((*f)->target == (*n)))
       {
-	sametarget = true;
-	delArc=f;
+        sametarget = true;
+        delArc=f;
       }
     }
-        
+    
+    // if such an arc was found, save its weight, delete it and add its weight to the arc from t12 to postset of t2
     if (sametarget)
     {
       int weightsave = arc_weight(t12,(*n));
@@ -917,12 +955,19 @@ void PetriNet::mergeTransitions(Transition *t1, Transition *t2)
       newArc(t12, (*n), STANDARD, (arc_weight(t2,(*n)) + weightsave));
       sametarget = false;
     }
+    // if no such arc was found, simply add an arc from t12 to postset of t2
     else
     {
       newArc(t12, (*n), STANDARD, arc_weight(t2,(*n)));
     }
   }
-
+  
+  
+  // set max_occurrences
+  t12->max_occurrences = (t1->max_occurrences > t2->max_occurrences) ?
+    t1->max_occurrences :
+    t2->max_occurrences;
+  
   removeTransition(t1);
   removeTransition(t2);
 }
@@ -956,65 +1001,79 @@ void PetriNet::mergeParallelTransitions(Transition *t1, Transition *t2)
 {
   if (t1 == t2)
     return;
-
+  
   assert(t1 != NULL);
   assert(t2 != NULL);
   
   Node *t12 = newTransition("");
-
+  
   // organize the communication type of the new transition
   if (t1->type == t2->type)
     t12->type = t1->type;
-
+  
   else if ((t1->type == IN && t2->type == INTERNAL) ||
-      (t1->type == INTERNAL && t2->type == IN))
+           (t1->type == INTERNAL && t2->type == IN))
     t12->type = IN;
-
+  
   else if ((t1->type == INTERNAL && t2->type == OUT) ||
-      (t1->type == OUT && t2->type == INTERNAL))
+           (t1->type == OUT && t2->type == INTERNAL))
     t12->type = OUT;
-
+  
   else if ((t1->type == OUT && t2->type == IN) ||
-      (t1->type == IN && t2->type == OUT) ||
-      (t1->type == INOUT || t2->type == INOUT))
+           (t1->type == IN && t2->type == OUT) ||
+           (t1->type == INOUT || t2->type == INOUT))
     t12->type = INOUT;
-
-
+  
+  
   // copy t1's history to t12
-  for (vector<string>::iterator role = t1->history.begin(); role != t1->history.end(); role++)
+  for (list<string>::iterator role = t1->history.begin(); role != t1->history.end(); role++)
   {
     roleMap[*role] = t12;
     t12->history.push_back(*role);
+    if (t1->prefix != "" || t2->prefix != "")
+    {
+      roleMap[t1->prefix + *role] = t12;
+      t12->history.push_back(t1->prefix + *role);
+      roleMap[t2->prefix + *role] = t12;
+      t12->history.push_back(t2->prefix + *role);
+    }
   }
-
+  
   // copy t2's history to t12
-  for (vector<string>::iterator role = t2->history.begin(); role != t2->history.end(); role++)
+  for (list<string>::iterator role = t2->history.begin(); role != t2->history.end(); role++)
   {
     roleMap[*role] = t12;
     t12->history.push_back(*role);
+    if (t1->prefix != "" || t2->prefix != "")
+    {
+      roleMap[t1->prefix + *role] = t12;
+      t12->history.push_back(t1->prefix + *role);
+      roleMap[t2->prefix + *role] = t12;
+      t12->history.push_back(t2->prefix + *role);
+    }
   }
-
+  
   // merge pre- and postsets for t12
   t12->preset=setUnion(t1->preset, t2->preset);
   t12->postset=setUnion(t1->postset, t2->postset);
-
+  
   // create the weighted arcs for t12
   set<Node *> preset2without1 = setDifference(t12->preset,t1->preset);
   set<Node *> postset2without1 = setDifference(t12->postset,t1->postset);
-
-
+  
+  
   for (set<Node *>::iterator n = t1->preset.begin(); n != t1->preset.end(); n++)
     newArc((*n), t12, STANDARD, arc_weight((*n),t1));
-
+  
   for (set<Node *>::iterator n = preset2without1.begin(); n != preset2without1.end(); n++)
     newArc((*n), t12, STANDARD, arc_weight((*n),t2));
-
+  
   for (set<Node *>::iterator n = t1->postset.begin(); n != t1->postset.end(); n++)
     newArc(t12, (*n), STANDARD, arc_weight(t1,(*n)));
-
+  
   for (set<Node *>::iterator n = postset2without1.begin(); n != postset2without1.end(); n++)
     newArc(t12, (*n), STANDARD, arc_weight(t2,(*n)));
-
+  
   removeTransition(t1);
   removeTransition(t2);
 }
@@ -1048,62 +1107,78 @@ void PetriNet::mergePlaces(Place * & p1, Place * & p2)
 {
   if (p1 == p2 && p1 != NULL)
     return;
-
+  
   if (p1 == NULL || p2 == NULL)
   {
     std::cerr << "[PN] At least one parameter of mergePlaces was NULL!" << std::endl;
     assert(false);
     return;
   }
-
+  
   assert(p1 != NULL);
   assert(p2 != NULL);
   assert(p1->type == INTERNAL);
   assert(p2->type == INTERNAL);
-
+  
   Place *p12 = newPlace("");
-
+  
   p12->tokens = max(p1->tokens, p2->tokens);
   p12->isFinal = (p1->isFinal || p2->isFinal);
   p12->wasExternal = p1->wasExternal + p2->wasExternal;
-
-  for (vector<string>::iterator role = p1->history.begin(); role != p1->history.end(); role++)
+  
+  for (list<string>::iterator role = p1->history.begin(); role != p1->history.end(); role++)
   {
     p12->history.push_back(*role);
     roleMap[*role] = p12;
+    if (p1->prefix != "" || p2->prefix != "")
+    {
+      roleMap[p1->prefix + *role] = p12;
+      p12->history.push_back(p1->prefix + *role);
+      roleMap[p2->prefix + *role] = p12;
+      p12->history.push_back(p2->prefix + *role);
+    }
   }
-
-  for (vector<string>::iterator role = p2->history.begin(); role != p2->history.end(); role++)
+  
+  for (list<string>::iterator role = p2->history.begin(); role != p2->history.end(); role++)
   {
     p12->history.push_back(*role);
     roleMap[*role] = p12;
+    if (p1->prefix != "" || p2->prefix != "")
+    {
+      roleMap[p1->prefix + *role] = p12;
+      p12->history.push_back(p1->prefix + *role);
+      roleMap[p2->prefix + *role] = p12;
+      p12->history.push_back(p2->prefix + *role);
+    }
   }
-
+  
   // merge pre- and postsets for p12
   p12->preset=setUnion(p1->preset, p2->preset);
   p12->postset=setUnion(p1->postset, p2->postset);
-
+  
   // create the weighted arcs for p12
-
+  
   set<Node *> preset2without1 = setDifference(p12->preset,p1->preset);
   set<Node *> postset2without1 = setDifference(p12->postset,p1->postset);
-
-
+  
+  if ((p1->preset.size() + p1->postset.size()) > 1000)
+    std::cerr << (p1->preset.size() + p1->postset.size()) << " arcs to add..." << std::endl;
+  
   for (set<Node *>::iterator n = p1->preset.begin(); n != p1->preset.end(); n++)
     newArc((*n), p12, STANDARD, arc_weight((*n),p1));
-
+  
   for (set<Node *>::iterator n = preset2without1.begin(); n != preset2without1.end(); n++)
     newArc((*n), p12, STANDARD, arc_weight((*n),p2));
-
+  
   for (set<Node *>::iterator n = p1->postset.begin(); n != p1->postset.end(); n++)
     newArc(p12, (*n), STANDARD, arc_weight(p1,(*n)));
-
+  
   for (set<Node *>::iterator n = postset2without1.begin(); n != postset2without1.end(); n++)
     newArc(p12, (*n), STANDARD, arc_weight(p2,(*n)));
-
+  
   removePlace(p1);
   removePlace(p2);
-
+  
   p1 = NULL;
   p2 = NULL;
 }
@@ -1193,11 +1268,12 @@ void PetriNet::renamePlace(string old_name, string new_name)
   assert(p != NULL);
 
   bool found = false;
-  for (unsigned int i = 0; i < p->history.size(); i++)
+
+  for( list<string>::iterator iter = p->history.begin(); iter != p->history.end(); iter ++)
   {
-    if (p->history[i] == old_name)
+    if ( *iter == old_name )
     {
-      p->history[i] = new_name;
+      *iter = new_name;
       found = true;
       break;
     }
@@ -1224,12 +1300,22 @@ void PetriNet::renamePlace(string old_name, string new_name)
  */
 unsigned int PetriNet::arc_weight(Node *my_source, Node *my_target) const
 {
+
+  map< pair<Node*, Node*>, int >::const_iterator iter = weight.find(pair<Node*, Node*>(my_source, my_target));
+
+  assert( iter != weight.end() ) ;
+  assert( iter->second > 0 );
+
+  return iter->second;
+
+  /*
   for (set<Arc *>::iterator f = F.begin(); f != F.end(); f++)
     if (((*f)->source == my_source) && ((*f)->target == my_target))
       return (*f)->weight;
 
   assert(false);
   return 1;
+  */
 }
 
 
@@ -1355,7 +1441,7 @@ set<Node *> PetriNet::postset(Node *n) const
 Place *PetriNet::findPlace(string role) const
 {
   map< std::string, Node* >::const_iterator p = roleMap.find(role);
-
+  
   if ((p != roleMap.end()) && (p->second->nodeType == PLACE))
     return (static_cast<Place *>(p->second));
   else
@@ -1364,10 +1450,10 @@ Place *PetriNet::findPlace(string role) const
     {
       map< std::string, Node* >::const_iterator p_with_suffix = roleMap.find(role + "." + forEach_suffix[0]);
       if ((p_with_suffix != roleMap.end()) && (p_with_suffix->second->nodeType == PLACE))
-	return (static_cast<Place *>(p_with_suffix->second));
+        return (static_cast<Place *>(p_with_suffix->second));
     }
   }
-
+  
   return NULL;
 }
 
@@ -1407,9 +1493,14 @@ Place *PetriNet::findPlace(unsigned int id, string role) const
 Transition *PetriNet::findTransition(string role) const
 {
   map< std::string, Node* >::const_iterator t = roleMap.find(role);
-
-  if ((t != roleMap.end()) && (t->second->nodeType == TRANSITION))
-    return (static_cast<Transition *>(t->second));
+  
+  if (t != roleMap.end() && t->second != NULL )
+  {
+    if (t->second->nodeType == TRANSITION)
+      return (dynamic_cast<Transition *>(t->second));
+    else
+      return NULL;
+  }
   else
     return NULL;
 }
@@ -1468,10 +1559,10 @@ unsigned int PetriNet::getId()
 void PetriNet::add_interface_suffix(string suffix)
 {
   for (set<Place *>::const_iterator place = P_in.begin(); place != P_in.end(); place ++)
-    (*place)->history[0] += suffix;
+    *(*place)->history.begin() += suffix;
 
   for (set<Place *>::const_iterator place = P_out.begin(); place != P_out.end(); place ++)
-    (*place)->history[0] += suffix;
+    *(*place)->history.begin() += suffix;
 }
 
 
@@ -1490,7 +1581,7 @@ void PetriNet::addPrefix(string prefix)
   {
     (*place)->prefix = prefix;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
     {
       roleMap[(prefix + *name)] = *place;
     }
@@ -1499,7 +1590,7 @@ void PetriNet::addPrefix(string prefix)
   {
     (*place)->prefix = prefix;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
     {
       roleMap[(prefix + *name)] = *place;
     }
@@ -1508,7 +1599,7 @@ void PetriNet::addPrefix(string prefix)
   {
     (*place)->prefix = prefix;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
     {
       roleMap[(prefix + *name)] = *place;
     }
@@ -1518,7 +1609,7 @@ void PetriNet::addPrefix(string prefix)
   {
     (*transition)->prefix = prefix;
 
-    for(vector< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
+    for(list< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
       roleMap[prefix + *name] = *transition;
   }
 }
@@ -1551,7 +1642,7 @@ void PetriNet::compose(const PetriNet &net)
 
     roleMap[ newPlace->nodeFullName() ] = newPlace;
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[(newPlace->prefix + *name)] = newPlace;
   }
 
@@ -1572,7 +1663,7 @@ void PetriNet::compose(const PetriNet &net)
     }
     // else, we don't need to add the place, it already exist (arcs are connected with the existing place)
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[(newPlace->prefix + *name)] = newPlace;
   }
 
@@ -1593,7 +1684,7 @@ void PetriNet::compose(const PetriNet &net)
     }
     // else, we don't need to add the place, it already exist (arcs are connected with the existing place)
 
-    for(vector< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
+    for(list< string >::iterator name = (*place)->history.begin(); name != (*place)->history.end(); name++)
       roleMap[(newPlace->prefix + *name)] = newPlace;
   }
 
@@ -1608,7 +1699,7 @@ void PetriNet::compose(const PetriNet &net)
 
     roleMap[ newTransition->nodeFullName() ] = newTransition;
 
-    for(vector< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
+    for(list< string >::iterator name = (*transition)->history.begin(); name != (*transition)->history.end(); name++)
       roleMap[ newTransition->prefix + *name ] = newTransition;
   }
 
@@ -1654,10 +1745,14 @@ void PetriNet::compose(const PetriNet &net)
       if ( (*place)->prefix != (oPlace)->prefix )
       {
         (*place)->type = INTERNAL;
-        (*place)->history[0] = (*place)->nodeFullName();
+        if ( (*place)->max_occurrences < (oPlace)->max_occurrences )
+        {
+          (*place)->max_occurrences = (oPlace)->max_occurrences;
+        }
+        (*place)->history.push_front( (*place)->nodeFullName() );
         roleMap[(*place)->nodeFullName()] = (*place);
         (oPlace)->type = INTERNAL;
-        (oPlace)->history[0] = (oPlace)->nodeFullName();
+        (oPlace)->history.push_front( (oPlace)->nodeFullName() );
         roleMap[(oPlace)->nodeFullName()] = (oPlace);
         (oPlace)->wasExternal = (oPlace)->nodeFullName().substr((oPlace)->nodeFullName().find_first_of(".") + 1);
         P.insert(*place);
@@ -1769,61 +1864,75 @@ void PetriNet::reenumerate()
 void PetriNet::calculate_max_occurrences()
 {
 #ifdef USING_BPEL2OWFN
+  
+  // set the max_occurrences for the transitions (should make more sense...)
+  for (set<Transition *>::iterator transition = T.begin();
+       transition != T.end();
+       transition++)
+  {
+    unsigned int activity_identifier = toUInt((*(*transition)->history.begin()).substr(0, (*(*transition)->history.begin()).find_first_of(".")));
+    
+    assert(globals::ASTEmap[activity_identifier] != NULL);
+    (*transition)->max_occurrences = globals::ASTEmap[activity_identifier]->max_occurrences;
+  }
+  
+  
+  
   // process the input places
   for (set<Place *>::iterator p = P_in.begin(); p != P_in.end(); p++)
   {
     set<unsigned int> receiving_activities;
-
+    
     for (set<Node *>::iterator t = (*p)->postset.begin(); t != (*p)->postset.end(); t++)
       for (unsigned int i = 0; i < (*t)->history.size(); i++)
       {
-	// unsigned int transition_activity_id = toUInt((*t)->history[i].substr(0, (*t)->history[i].find_first_of(".")));
+        // unsigned int transition_activity_id = toUInt((*t)->history[i].substr(0, (*t)->history[i].find_first_of(".")));
         string act_id = (*t)->nodeFullName().substr(0, (*t)->nodeFullName().find_first_of("."));
-
+        
         act_id = act_id.substr( act_id.find_last_of( "_" ) + 1 );
-	unsigned int transition_activity_id = toUInt( act_id );
-	assert(globals::ASTEmap[transition_activity_id] != NULL);
-	receiving_activities.insert(transition_activity_id);
+        unsigned int transition_activity_id = toUInt( act_id );
+        assert(globals::ASTEmap[transition_activity_id] != NULL);
+        receiving_activities.insert(transition_activity_id);
       }
-
-    for (set<unsigned int>::iterator activity_id = receiving_activities.begin(); activity_id != receiving_activities.end(); activity_id++)
-    {
-      if (globals::ASTEmap[*activity_id]->max_occurrences == UINT_MAX)
-	(*p)->max_occurrences = UINT_MAX;
-      else if ((*p)->max_occurrences != UINT_MAX)
-      {
-	if (globals::ASTEmap[*activity_id]->activityTypeName() == "repeatUntil" ||
-	    globals::ASTEmap[*activity_id]->activityTypeName() == "while")
-	  (*p)->max_occurrences += globals::ASTEmap[*activity_id]->max_loops;
-	else
-	  (*p)->max_occurrences += globals::ASTEmap[*activity_id]->max_occurrences;
-      }
-//  Debugging Help
-//  cerr << "gehoert auch zu Aktivitaet: " << globals::ASTEmap[*activity_id]->activityTypeName() << "!\n";
-    }
+        
+        for (set<unsigned int>::iterator activity_id = receiving_activities.begin(); activity_id != receiving_activities.end(); activity_id++)
+        {
+          if (globals::ASTEmap[*activity_id]->max_occurrences == UINT_MAX)
+            (*p)->max_occurrences = UINT_MAX;
+          else if ((*p)->max_occurrences != UINT_MAX)
+          {
+            if (globals::ASTEmap[*activity_id]->activityTypeName() == "repeatUntil" ||
+                globals::ASTEmap[*activity_id]->activityTypeName() == "while")
+              (*p)->max_occurrences += globals::ASTEmap[*activity_id]->max_loops;
+            else
+              (*p)->max_occurrences += globals::ASTEmap[*activity_id]->max_occurrences;
+          }
+          //  Debugging Help
+          //  cerr << "gehoert auch zu Aktivitaet: " << globals::ASTEmap[*activity_id]->activityTypeName() << "!\n";
+        }
   }
-
-  // process the output places
-  for (set<Place *>::iterator p = P_out.begin(); p != P_out.end(); p++)
-  {
-    set<unsigned int> sending_activities;
-
-    for (set<Node *>::iterator t = (*p)->preset.begin(); t != (*p)->preset.end(); t++)
-      for (unsigned int i = 0; i < (*t)->history.size(); i++)
-      {
-	unsigned int transition_activity_id = toUInt((*t)->history[i].substr(0, (*t)->history[i].find_first_of(".")));
-	assert(globals::ASTEmap[transition_activity_id] != NULL);
-	sending_activities.insert(transition_activity_id);
-      }
-
-    for (set<unsigned int>::iterator activity_id = sending_activities.begin(); activity_id != sending_activities.end(); activity_id++)
+    
+    // process the output places
+    for (set<Place *>::iterator p = P_out.begin(); p != P_out.end(); p++)
     {
-      if (globals::ASTEmap[*activity_id]->max_occurrences == UINT_MAX)
-	(*p)->max_occurrences = UINT_MAX;
-      else if ((*p)->max_occurrences != UINT_MAX)
-	(*p)->max_occurrences += globals::ASTEmap[*activity_id]->max_occurrences;
+      set<unsigned int> sending_activities;
+      
+      for (set<Node *>::iterator t = (*p)->preset.begin(); t != (*p)->preset.end(); t++)
+        for (list<string>::iterator iter = (*t)->history.begin(); iter != (*t)->history.end(); iter++)
+        {
+          unsigned int transition_activity_id = toUInt(iter->substr(0, iter->find_first_of(".")));
+          assert(globals::ASTEmap[transition_activity_id] != NULL);
+          sending_activities.insert(transition_activity_id);
+        }
+          
+          for (set<unsigned int>::iterator activity_id = sending_activities.begin(); activity_id != sending_activities.end(); activity_id++)
+          {
+            if (globals::ASTEmap[*activity_id]->max_occurrences == UINT_MAX)
+              (*p)->max_occurrences = UINT_MAX;
+            else if ((*p)->max_occurrences != UINT_MAX)
+              (*p)->max_occurrences += globals::ASTEmap[*activity_id]->max_occurrences;
+          }
     }
-  }
 #endif
 }
 
@@ -1900,14 +2009,14 @@ void PetriNet::produce(const PetriNet &net)
     if ( (*t)->labels.empty())
     {
       Transition *t_new = newTransition((*t)->nodeName());
-
+      
       // copy the arcs of the constraint oWFN
       for (set< Arc * >::iterator arc = net.F.begin(); arc != net.F.end(); arc ++)
       {
-	if ( ( (*arc)->source->nodeType == PLACE) && ( (*arc)->target == static_cast<Node *>(*t) ) )
-	  newArc( findPlace( (*arc)->source->nodeName() ), t_new, STANDARD, (*arc)->weight );
-	if ( ( (*arc)->target->nodeType == PLACE) && ( (*arc)->source == static_cast<Node *>(*t) ) )
-	  newArc( t_new, findPlace( (*arc)->target->nodeName() ), STANDARD, (*arc)->weight );
+        if ( ( (*arc)->source->nodeType == PLACE) && ( (*arc)->target == static_cast<Node *>(*t) ) )
+          newArc( findPlace( (*arc)->source->nodeName() ), t_new, STANDARD, (*arc)->weight );
+        if ( ( (*arc)->target->nodeType == PLACE) && ( (*arc)->source == static_cast<Node *>(*t) ) )
+          newArc( t_new, findPlace( (*arc)->target->nodeName() ), STANDARD, (*arc)->weight );
       }
     }
   }
@@ -1931,36 +2040,36 @@ void PetriNet::produce(const PetriNet &net)
     for (set<string>::iterator label = (*t)->labels.begin(); label != (*t)->labels.end(); label++)
     {
       Transition *t_l = findTransition(*label);
-
+      
       // specified transition not found -- trying places instead
       if (t_l == NULL)
       {
-	Place *p = findPlace(*label);
-	if (p != NULL)
-	{
-	  set<Node *> transitions_p;
-
-	  if (p->type == IN)
-	    transitions_p = p->postset;
-	  else
-	    transitions_p = p->preset;
-
-	  for (set<Node *>::iterator pre_transition = transitions_p.begin(); pre_transition != transitions_p.end(); pre_transition++)
-	  {
-	    used_labels.insert((*pre_transition)->nodeName());
-    	    transition_pairs.insert(pair<Transition *, Transition *>(static_cast<Transition *>(*pre_transition), *t));
-	  }
-	}
-	else
-	{
-	  std::cerr << "label " << *label << " neither describes a transition nor a place" << std::endl;
-	  assert(false);
-	}
+        Place *p = findPlace(*label);
+        if (p != NULL)
+        {
+          set<Node *> transitions_p;
+          
+          if (p->type == IN)
+            transitions_p = p->postset;
+          else
+            transitions_p = p->preset;
+          
+          for (set<Node *>::iterator pre_transition = transitions_p.begin(); pre_transition != transitions_p.end(); pre_transition++)
+          {
+            used_labels.insert((*pre_transition)->nodeName());
+            transition_pairs.insert(pair<Transition *, Transition *>(static_cast<Transition *>(*pre_transition), *t));
+          }
+        }
+        else
+        {
+          std::cerr << "label " << *label << " neither describes a transition nor a place" << std::endl;
+          assert(false);
+        }
       }
       else
       {
-	used_labels.insert(*label);
-	transition_pairs.insert(pair<Transition *, Transition *>(t_l, *t));
+        used_labels.insert(*label);
+        transition_pairs.insert(pair<Transition *, Transition *>(t_l, *t));
       }
     }
   }
@@ -1977,18 +2086,18 @@ void PetriNet::produce(const PetriNet &net)
     for (set< Arc * >::iterator arc = net.F.begin(); arc != net.F.end(); arc ++)
     {
       if ( ( (*arc)->source->nodeType == PLACE) && ( (*arc)->target == static_cast<Node *>(tp->second) ) )
-	newArc( findPlace( (*arc)->source->nodeName() ), t_new, STANDARD, (*arc)->weight );
+        newArc( findPlace( (*arc)->source->nodeName() ), t_new, STANDARD, (*arc)->weight );
       if ( ( (*arc)->target->nodeType == PLACE) && ( (*arc)->source == static_cast<Node *>(tp->second) ) )
-	newArc( t_new, findPlace( (*arc)->target->nodeName() ), STANDARD, (*arc)->weight );
+        newArc( t_new, findPlace( (*arc)->target->nodeName() ), STANDARD, (*arc)->weight );
     }
-
+    
     // copy the arcs of the oWFN
     for (set< Arc * >::iterator arc = F.begin(); arc != F.end(); arc ++)
     {
       if ( ( (*arc)->source->nodeType == PLACE) && ( (*arc)->target == static_cast<Node *>(tp->first) ) )
-	newArc( findPlace( (*arc)->source->nodeName() ), t_new, STANDARD, (*arc)->weight );
+        newArc( findPlace( (*arc)->source->nodeName() ), t_new, STANDARD, (*arc)->weight );
       if ( ( (*arc)->target->nodeType == PLACE) && ( (*arc)->source == static_cast<Node *>(tp->first) ) )
-	newArc( t_new, findPlace( (*arc)->target->nodeName() ), STANDARD, (*arc)->weight );
+        newArc( t_new, findPlace( (*arc)->target->nodeName() ), STANDARD, (*arc)->weight );
     }
   }
 
@@ -2002,6 +2111,24 @@ void PetriNet::produce(const PetriNet &net)
 
 
 
+/*!
+ * \brief  adds a transition that has read arcs to all final places
+ */
+void PetriNet::loop_final_state()
+{
+  Transition *t = newTransition("antideadlock"); 
+  
+  for (set<Place *>::iterator place = P.begin(); place != P.end(); place++)
+  {
+    if ( (*place)->isFinal )
+      newArc(t, *place, READ);
+  }
+}
+
+
+
+
+
 /******************************************************************************
  * Functions to manage <forEach> suffix
  *****************************************************************************/
