@@ -31,13 +31,13 @@
  *
  * \since   2005/10/18
  *
- * \date    \$Date: 2007/05/10 12:07:17 $
+ * \date    \$Date: 2007/06/07 12:28:59 $
  *
  * \note    This file is part of the tool GNU BPEL2oWFN and was created during
  *          the project Tools4BPEL at the Humboldt-Universitt zu Berlin. See
  *          http://www.informatik.hu-berlin.de/top/tools4bpel for details.
  *
- * \version \$Revision: 1.147 $
+ * \version \$Revision: 1.152 $
  *
  * \ingroup petrinet
  */
@@ -62,6 +62,7 @@
 #include <vector>
 #include <set>
 #include <map>
+#include <list>
 #include <deque>
 
 using std::string;
@@ -69,6 +70,8 @@ using std::vector;
 using std::deque;
 using std::set;
 using std::map;
+using std::list;
+using std::pair;
 using std::ostream;
 
 
@@ -195,7 +198,7 @@ class Node
 
   protected:
     /// the set of roles (i.e. the history) of the node
-    vector<string> history;
+    list<string> history;
 
     /// the id of the node
     unsigned int id;
@@ -212,6 +215,9 @@ class Node
     /// the postset of this node
     set<Node*> postset;
 
+    /// the maximal occurrences of this (communication) place or transition
+    unsigned int max_occurrences;
+
 
     /// true if first role contains role
     bool firstMemberAs(string role) const;
@@ -308,9 +314,6 @@ class Place: public Node
     /// initial marking of the place
     unsigned int tokens;
 
-    /// the maximal occurrences of this (communication) place
-    unsigned int max_occurrences;
-
     /// not empty if place was once an communication place and is now internal because of choreographie
     string wasExternal;
 
@@ -449,6 +452,8 @@ class PetriNet
     /// rename a node (i.e., rename one role in its history)
     void renamePlace(string old_name, string new_name);
 
+    /// removes a transition from the net
+    void removeTransition(Transition *t);    
 
     /// applies structral reduction rules
     unsigned int reduce(unsigned int reduction_level = 5);
@@ -474,6 +479,9 @@ class PetriNet
     /// re-enumerates the nodes
     void reenumerate();
 
+    /// add a loop to the final states to check deadlock freedom with LoLA
+    void loop_final_state();
+    
     /// statistical output
     string information() const;
 
@@ -518,9 +526,6 @@ class PetriNet
     /// removes a place from the net
     void removePlace(Place *p);
 
-    /// removes a transition from the net
-    void removeTransition(Transition *t);
-
     /// removes an arc from the net
     void removeArc(Arc *f);
 
@@ -592,6 +597,9 @@ class PetriNet
     /// elimination of identical places
     void reduce_equal_places();
 
+    /// remove unneeded initially marked places in choreographies
+    void reduce_remove_initially_marked_places_in_choreographies();
+
 
     /// returns an id for new nodes
     unsigned int getId();
@@ -628,6 +636,9 @@ class PetriNet
 
     /// mapping of roles to nodes of the Petri net
     map<string, Node *> roleMap;
+
+    /// mapping of arcs to their appropriate weight
+    map< pair< Node*, Node* >, int > weight;
 };
 
 
